/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.io.*;
import java.util.*;

public class ArticleRecord extends PalmRecord {
    public int flags = 0;
    public Date rcvDate = null;
    public int xPostings = 0;
    public int server = 0;
    public String from = null;
    public String date = null;
    public String subject = null;
    public String newsgroups = null;
    public String msgid = null;
    public String ref = null;
    public String fup_to = null;
    public String headers = null;
    public String body = null;

    static final int FLAG_MARKED = 1;
    static final int FLAG_READ = 2;
    static final int FLAG_KEEP = 4;
    static final int FLAG_DONT_REINDEX = 8;

    public ArticleRecord() { setId(0); }

    public boolean isMarked() { return (flags & FLAG_MARKED) != 0; }
    public boolean isRead() { return (flags & FLAG_READ) != 0; }
    public boolean isKeep() { return (flags & FLAG_KEEP) != 0; }
    public boolean isDontReindex() { return (flags & FLAG_DONT_REINDEX) != 0; }

    public void writeData(DataOutputStream out) throws IOException{
        out.writeShort(this.flags);
        if (rcvDate != null) out.writeInt(dateToPalmSec(this.rcvDate)); else out.writeInt(0);
        out.writeShort(this.xPostings);
        out.writeShort(this.server);
  
        if (from != null) {
            out.write(from.getBytes());
        }
        out.write(0);

        if (date != null) {
            out.write(date.getBytes());
        }
        out.write(0);

        if (subject != null) {
            out.write(subject.getBytes());
        }
        out.write(0);

        if (newsgroups != null) {
            out.write(newsgroups.getBytes());
        }
        out.write(0);

        if (msgid != null) {
            out.write(msgid.getBytes());
        }
        out.write(0);

        if (ref != null) {
            out.write(ref.getBytes());
        }
        out.write(0);

        if (fup_to != null) {
            out.write(fup_to.getBytes());
        }
        out.write(0);

        if (headers != null) {
            out.write(headers.getBytes());
        }
        out.write(0);

        if (body != null) {
            out.write(body.getBytes());
        }
        out.write(0);
    }

    public void readData(DataInputStream in) throws IOException{
        this.flags = in.readUnsignedShort();
        this.rcvDate = palmSecToDate(in.readInt());
        this.xPostings = in.readUnsignedShort();
        this.server = in.readUnsignedShort();
        this.from = readCString(in);
        this.date = readCString(in);
        this.subject = readCString(in);
        this.newsgroups = readCString(in);
        this.msgid = readCString(in);
        this.ref = readCString(in);
        this.fup_to = readCString(in);
        this.headers = readCString(in);
        this.body = readCString(in);
    }

    /** Returns a string representation of the object */
    public String toString() {
        return "News article record: " + this.msgid + " " + super.toString();
    }

    /** Returns a formatted string representation of the object */
    public String toFormattedString() {
        return ("News article record: {\r\n" +
                "  flags: " + this.flags + "\r\n" +
                "  rcvDate: " + this.rcvDate + "\r\n" +
                "  xPostings: " + this.xPostings + "\r\n" +
                "  server: " + this.server + "\r\n" +
                "  from: " + this.from + "\r\n" +
                "  date: " + this.date + "\r\n" +
                "  subject: " + this.subject + "\r\n" +
                "  newsgroups: " + this.newsgroups + "\r\n" +
                "  msgid: " + this.msgid + "\r\n" +
                "  ref: " + this.ref + "\r\n" +
                "  fup_to: " + this.fup_to + "\r\n" +
                "  headers: " + this.headers + "\r\n" +
                "  body: " + this.body + "\r\n" +
                 "}\r\n" +
                super.toFormattedString());
    }
}


