/**
 * Title:	 Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author	 Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;


public class ConduitConfigure implements WindowListener, ActionListener {

    public final int MAX_SERVERS = 10;
    public final int CONFIG_VERSION = 0x0009020a;

    public final static int DEBUG_SKIPNORMAL = 256;
    public final static int DEBUG_DUMPALL = 512;
    public final static int DEBUG_SPECIAL = 1024;

    public final static int NO_PURGE = -1;
    public final static int READ_MODE_NORMAL = 0;

    public boolean onceADay;
    public boolean skipSync;
    public int debugLevel;
    public int purgeMsgidDays;	// since version 0x00090207
    public int purgeArticleDays; // since version 0x00090207
    public int readMode; // not implemented
    public boolean allowThreading; // since version 0x00090208
    public Date lastSync;
    public int numServers;

    public String[] newsrcPath;
    public boolean[] useNewsrc;
    public String[] server;

    public boolean usePollPrefs; // since version 0x00090209
    public boolean useModeReader; // since version 0x009020a

    private String junkText = "";

    /** Path to the configuration properties file on the PC
    */
    private String propsPath;

    /** Name of the conduit
    */
    public String name;

    /** The Configuration dialog
    */
    private Dialog configureDialog;

    private Checkbox cbSkipSync;
    private Checkbox cbOnceADay;

    private Checkbox cbPurgeMsgid;
    private TextField tfPurgeMsgid;

    private Checkbox cbPurgeArticle;
    private TextField tfPurgeArticle;

    private Checkbox cbAllowThreading;
    private Checkbox cbUsePollPrefs;
    private Checkbox cbUseModeReader;

    private TextField tfDebugLevel;
    private Checkbox[] cbUseNewsrc;
    private TextField[] tfnewsrcPath;

    /** Constructor. Takes the name of the conduit
    *	and the path to the configuration file as arguments
    */
    public ConduitConfigure(String name, String propsPath) {
	this.name = name;
	this.propsPath = propsPath;

	newsrcPath = new String[MAX_SERVERS];
	useNewsrc = new boolean[MAX_SERVERS];
	server = new String[MAX_SERVERS];

	tfnewsrcPath = new TextField[MAX_SERVERS];
	cbUseNewsrc = new Checkbox[MAX_SERVERS];
    }

    /** Creates the configuration dialog box.
    */
    public int createDialog() {

	Dimension dim;

	Frame f = new Frame();

	// Create a dialog
	configureDialog = new Dialog(f, name);
	configureDialog.setResizable(true);
	configureDialog.setModal(true);
	configureDialog.setBackground(SystemColor.control);

	configureDialog.setLayout(new GridBagLayout());

	// Create a constraints object, and specify some default values
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;   // components grow in both dimensions
	c.insets = new Insets(5,5,5,5); // 5-pixel margins on all sides

	// Create the components
	cbSkipSync = new Checkbox("Skip Yanoff synchronisation", skipSync);
	cbOnceADay = new Checkbox("Sync only once a day", onceADay);
	cbPurgeMsgid = new Checkbox("Purge MsgIDs older than (days)", (purgeMsgidDays != NO_PURGE));
	tfPurgeMsgid = new TextField((purgeMsgidDays != NO_PURGE) ? Integer.toString(purgeMsgidDays) : "", 20);
	cbPurgeArticle = new Checkbox("UNIMPLEMENTED! Purge articles older than (days)", (purgeArticleDays != NO_PURGE));
	tfPurgeArticle = new TextField((purgeArticleDays != NO_PURGE) ? Integer.toString(purgeArticleDays) : "", 20);
	cbAllowThreading = new Checkbox("Allow threading by conduit", allowThreading);
	cbUsePollPrefs = new Checkbox("EXPERIMENTAL! Use Poll Prefs", usePollPrefs);
	cbUseModeReader = new Checkbox("Send MODE READER to news server", useModeReader);

	tfDebugLevel = new TextField(Integer.toString(debugLevel), 20);

	Label[] pathLabel = new Label[MAX_SERVERS];
	for (int i=0; i<MAX_SERVERS; i++) {
	    cbUseNewsrc[i] = new Checkbox("Use newsrc for server "+server[i], useNewsrc[i]);

	    tfnewsrcPath[i] = new TextField(newsrcPath[i], 40);
	    pathLabel[i] = new Label("Newsrc path", Label.LEFT);
	}

	Button bok = new Button("OK"); bok.addActionListener(this);
	Button bcancel = new Button("Cancel"); bcancel.addActionListener(this);

	int line=-1;

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(cbSkipSync, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(cbOnceADay, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 0.1; c.weighty = 0.0;
	configureDialog.add(cbPurgeMsgid, c);

	c.gridx = 1; c.gridy = line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(tfPurgeMsgid, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 0.1; c.weighty = 0.0;
	configureDialog.add(cbPurgeArticle, c);

	c.gridx = 1; c.gridy = line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(tfPurgeArticle, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 0.1; c.weighty = 0.0;
	configureDialog.add(cbAllowThreading, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 0.1; c.weighty = 0.0;
	configureDialog.add(cbUsePollPrefs, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 0.1; c.weighty = 0.0;
	configureDialog.add(cbUseModeReader, c);

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(new Label("Debug level", Label.LEFT), c);

	c.gridx = 1; c.gridy = line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(tfDebugLevel, c);

	for (int i=0; i<numServers; i++) {
	    c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	    c.weightx = 1.0; c.weighty = 0.0;
	    configureDialog.add(cbUseNewsrc[i], c);

	    c.gridx = 0; c.gridy = ++line; c.gridwidth = 1; c.gridheight = 1;
	    c.weightx = 1.0; c.weighty = 0.0;
	    configureDialog.add(pathLabel[i], c);

	    c.gridx = 1; c.gridy = line; c.gridwidth = 1; c.gridheight = 1;
	    c.weightx = 1.0; c.weighty = 0.0;
	    configureDialog.add(tfnewsrcPath[i], c);
	}

	c.gridx = 0; c.gridy = ++line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(bok, c);

	c.gridx = 1; c.gridy = line; c.gridwidth = 1; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(bcancel, c);

	TextArea debugInfo = new TextArea("Debug level: low is little info, high is MUCH info "+
	      "256=skip normal operation; 512=dump Yanoff databases to log; 1024=special",
	      3,1, TextArea.SCROLLBARS_NONE);
	debugInfo.setEditable(false);
	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 0.0;
	configureDialog.add(debugInfo, c);

	TextArea warning = new TextArea("WARNING: Running the conduit while "+
		"another program (like Netscape or Outlook Express) has the "+
		"newsrc file(s) open may cause unexpected results\n", 
		3,1, TextArea.SCROLLBARS_NONE);
	warning.setEditable(false);
	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 1.0;
	configureDialog.add(warning, c);

	Font font = warning.getFont();
	if (font != null)
	    junkText = junkText + "; Font: family "+warning.getFont().getFamily()+" name "+warning.getFont().getName();
	else
	    junkText = junkText + "; Font: null";
	junkText = junkText + warning.getFont();
	junkText = junkText + "\n" + Version.BUILD_DATE;
	Component comp=null;
	for (comp=warning; comp != null && comp.getFont() == null; comp = comp.getParent())
	    ;
	if (comp!=null) junkText = junkText + comp.getFont();

	TextArea junk = new TextArea(junkText, 
		3,1, TextArea.SCROLLBARS_NONE);
	junk.setEditable(false);
	c.gridx = 0; c.gridy = ++line; c.gridwidth = 2; c.gridheight = 1;
	c.weightx = 1.0; c.weighty = 1.0;
	configureDialog.add(junk, c);

	// Size the dialog
//	dim = configureDialog.getToolkit().getScreenSize();
//	  configureDialog.invalidate();
	configureDialog.setLocation(0,0);
//	  configureDialog.setSize(500, 600);
	configureDialog.validate();
//	configureDialog.setResizable(true);
	configureDialog.pack();
	configureDialog.addWindowListener(this);
//	configureDialog.setResizable(true);

	configureDialog.show();

	return 0;
    }

    /** Reads the configuration information from the PC's hard drive.
    *	
    */
    public void readProps() {

	String oldServer;   
	String newServer;

	try {
	    FileInputStream in = new FileInputStream(propsPath);
	    DataInputStream dataIn = new DataInputStream(new BufferedInputStream (in));

	    dataIn.mark(20);
	    int version = dataIn.readInt();
	    junkText = junkText + "Current version: " + CONFIG_VERSION +
		"; Config file version: " + version;
	    dataIn.reset();

	    if (version > CONFIG_VERSION+10) {
		throw new IOException("Unknown configuration file version");
	    }

	    if (version >= 0x00090205) {
		version = dataIn.readInt();
		skipSync = dataIn.readBoolean();
	    } else {
		skipSync = false;
	    }

	    if (version >= 0x00090206) {
		debugLevel = dataIn.readInt();
	    } else {
		debugLevel = 2;
	    }

	    if (version >= 0x00090207) {
		purgeMsgidDays = dataIn.readInt();
		purgeArticleDays = dataIn.readInt();
		readMode = dataIn.readInt();
	    } else {
		purgeMsgidDays = 5;
		purgeArticleDays = NO_PURGE;
		readMode = READ_MODE_NORMAL; // not implemented
	    }

	    if (version >= 0x00090208) {
		allowThreading = dataIn.readBoolean();
	    } else {
		allowThreading = false;
	    }

	    onceADay = dataIn.readBoolean();
	    lastSync = new Date(dataIn.readLong());
	    numServers = dataIn.readInt();

	    for (int i=0; i<MAX_SERVERS; i++) {
		server[i] = dataIn.readUTF();
		newsrcPath[i] = dataIn.readUTF();
		useNewsrc[i] = dataIn.readBoolean();
	    }

	    if (version >= 0x00090209) {
		usePollPrefs = dataIn.readBoolean();
	    } else {
		usePollPrefs = false;
	    }

	    if (version >= 0x0009020a) {
		useModeReader = dataIn.readBoolean();
	    } else {
		useModeReader = false;
	    }

	    dataIn.close();
	} catch (IOException e) {
	    junkText = junkText + "\n" + e.getMessage();
	    skipSync = false;
	    debugLevel = 2;
	    purgeMsgidDays = 5;
	    purgeArticleDays = NO_PURGE;
	    readMode = READ_MODE_NORMAL; // not implemented
	    allowThreading = false;
	    onceADay = false;
	    lastSync = new Date(0);
	    numServers = 1;

	    for (int i=0; i<MAX_SERVERS; i++) {
	      server[i] = "unknown";
	      newsrcPath[i] = "";
	      useNewsrc[i] = false;
	    }
	    usePollPrefs = false;
	    useModeReader = false;
	}
    }

    /** Write's the configuration information back to
    *	the PC's hard drive using the path passed in through
    *	the constructor.
    */
    public void writeProps() {

	try {
	    FileOutputStream out = new FileOutputStream(propsPath);
	    DataOutputStream dataOut = new DataOutputStream(out);

	    dataOut.writeInt(CONFIG_VERSION);		// since 00090205
	    dataOut.writeBoolean(skipSync);		// since 00090205
	    dataOut.writeInt(debugLevel);		// since 00090206
	    dataOut.writeInt(purgeMsgidDays);		// since 00090207
	    dataOut.writeInt(purgeArticleDays); 	// since 00090207
	    dataOut.writeInt(readMode); 		// since 00090207; not implemented
	    dataOut.writeBoolean(allowThreading);	// since 00090208

	    dataOut.writeBoolean(onceADay);
	    dataOut.writeLong(lastSync.getTime());
	    dataOut.writeInt(numServers);

	    for (int i=0; i<MAX_SERVERS; i++) {
	       dataOut.writeUTF(server[i]);
	       dataOut.writeUTF(newsrcPath[i]);
	       dataOut.writeBoolean(useNewsrc[i]);
	    }
	    dataOut.writeBoolean(usePollPrefs); 	// since 00090209
	    dataOut.writeBoolean(useModeReader);	 // since 0009020a

	    dataOut.close();
	} catch (IOException e) {
	    e.printStackTrace();
	    Log.abortSync();
	}
    }

    public void actionPerformed(ActionEvent e) {

	Button b = (Button)e.getSource();

	if(b.getLabel().equals("Cancel")) {
	    configureDialog.dispose();
	    return;
	} else if(b.getLabel().equals("Help")) {
	    // if help, pop up text window with help info

	} else if(b.getLabel().equals("OK")) {
	    skipSync = cbSkipSync.getState();
	    onceADay = cbOnceADay.getState();

	    if (cbPurgeMsgid.getState()) {
		try {
		    int temp = Integer.parseInt(tfPurgeMsgid.getText());
		    if (temp<0)
			return;
		    purgeMsgidDays = temp;
		} catch (NumberFormatException ex) {
		    return;
		}
	    } else purgeMsgidDays = NO_PURGE;

	    if (cbPurgeArticle.getState()) {
		try {
		    int temp = Integer.parseInt(tfPurgeArticle.getText());
		    if (temp<0)
			return;
		    purgeArticleDays = temp;
		} catch (NumberFormatException ex) {
		    return;
		}
	    } else purgeArticleDays = NO_PURGE;

	    allowThreading = cbAllowThreading.getState();

	    try {
		debugLevel = Integer.parseInt(tfDebugLevel.getText());
	    } catch (NumberFormatException ex) {
		return;
	    }
	    for (int i=0; i<MAX_SERVERS; i++) {
		newsrcPath[i] = tfnewsrcPath[i].getText();
		useNewsrc[i] = cbUseNewsrc[i].getState();
	    }
	    usePollPrefs = cbUsePollPrefs.getState();
	    useModeReader = cbUseModeReader.getState();
	    configureDialog.dispose();
	    return;
	}
    }

	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {
		// Dialog dismissed, don't save changes
		configureDialog.dispose();
	}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}

}
