/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class History extends PalmDatabase {
    static final String HISTORY_DB = "NewsHistory";
    static final String HISTORY_RECORD_CLASS = "org.vanbest.yanoff.HistoryRecord";

    public History(int creator, int type, int flags) throws IOException, ClassNotFoundException { 
        super(HISTORY_DB, creator, type, Class.forName(HISTORY_RECORD_CLASS), flags);
    }

    public void addMsgid(String msgid) throws IOException {
        HistoryRecord record = new HistoryRecord(new Date(System.currentTimeMillis()), msgid);
        addRecord(record);
    }

    public void removeMsgid (String msgid) throws IOException {
        boolean found = false;
	    PalmRecord record = null;

        for(record=firstRecord(); record != null && !found; record = nextRecord()) {
            found = msgid.equals(((HistoryRecord)record).msgid);
            if (found) deleteRecord(record);
        }
    }

    public boolean msgidExists(String msgid) {
        if (msgid == null) return false;
        boolean found = false;
    	PalmRecord record = null;

        for(record=firstRecord(); record != null && !found; record = nextRecord()) {
            found = msgid.equals(((HistoryRecord)record).msgid);
        }

        return found;
    }
 
    // Purge msgids older that <days> days from the cache
    void purge(int days) {
        Date now = new Date();
        for (int i=cache.size()-1; i>=0; i--) {
            HistoryRecord record = (HistoryRecord) cache.elementAt(i);
            if ((now.getTime() - record.date.getTime())/1000/60/60/24 >= days) {
                cache.removeElementAt(i);
            }
        }
    }

    public void cleanup(boolean doSort, boolean doPurge, int purgeDays) throws IOException {
        if (!doSort && !doPurge) return;

        Util.DEBUG(2, "Cleanup History ("+getRecordCount()+" records), starting at "+new Date(System.currentTimeMillis()).toLocaleString());

        Util.DEBUG(3, "    First read all records to cache");
        for (int i=0; i<getRecordCount(); i++) getRecordByIndex(i); // make sure all records are in the cache

	if (doPurge) {
            Util.DEBUG(3, "    Purging... "+new Date(System.currentTimeMillis()).toLocaleString());
            purge(purgeDays);
            Util.DEBUG(3, "    Finished purging, "+cache.size()+" records left "+new Date(System.currentTimeMillis()).toLocaleString());
        }

	if (doSort) {
            Util.DEBUG(3, "    Sorting at "+new Date(System.currentTimeMillis()).toLocaleString());
            Sorter.quicksort(cache);
            Util.DEBUG(3, "    Finished sorting "+new Date(System.currentTimeMillis()).toLocaleString());
        }

        // Remove all records from the Palm
        open();
        SyncManager.purgeAllRecs(DB);

        Util.DEBUG(3, "    Writing back to Palm "+new Date(System.currentTimeMillis()).toLocaleString());

        // And write them back in the proper order
        for (int i=0; i<cache.size(); i++) {
            HistoryRecord current=(HistoryRecord) cache.elementAt(i);
            current.setIndex(i);
            SyncManager.writeRec(DB, current);
        }

        recordCount = cache.size();

        Util.DEBUG(3, "Finished cleanup at "+new Date(System.currentTimeMillis()).toLocaleString());
    }

    public String toFormattedString() {
        String s = "";
        for (HistoryRecord record = (HistoryRecord) firstRecord(); record != null; record = (HistoryRecord) nextRecord()) {
            s = s + record.msgid + "\n";
        }
        return s;
    }


}


