/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.text.*;
import java.util.*;
import palm.conduit.*;

public class NetClient {
    /** 
      * This class services protocols like NNTP and SMTP
      */
    protected final String CRLF="\r\n";

    protected Socket socket;
    protected BufferedReader in;
    protected BufferedWriter out;
    protected String replyString;
    protected int reply;

    public NetClient() {}
    public NetClient(String host, int port) throws IOException, UnknownHostException {
        connect(host, port);
    }
 
    public void connect(String host, int port) throws IOException, UnknownHostException {
        socket = new Socket(host, port);
        in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        out = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream()));
    }

    public void disconnect() throws IOException {
        socket.close();
    }

    public int sendCommand(String command, String parameters) throws IOException {
        String commandString;
        if (parameters!=null && !parameters.equals(""))
    	    commandString = command+" "+parameters;
        else
	        commandString = command;

//        Log.out("-->"+command+" "+parameters);
        out.write(commandString+CRLF);
        out.flush();
        return readReply();
    }

    public int getReply() {
        return reply;
    }
  
    public String getReplyString() {
        return replyString;
    }

  /**
   * If there is a part ..<yyy>..., then take yyy as e-mail address.
   * Else, if there is a space, take everything up to the space as a-mail address
   * Else, take the whole input string
   */
   public static String extractEmailAddress(String fullAddress) {
        String address = "";
        boolean ready=false;
        int start=0;
        int end=0;

        start = fullAddress.indexOf("<");
        if (start != -1) {
            end = fullAddress.indexOf(">", start);
            if (end != -1) {
                address = fullAddress.substring(start+1, end);
                ready = true;
            }
        }
     
        end = fullAddress.indexOf(" ");
        if ( (!ready) && (end != -1) ) {
            address = fullAddress.substring(0, end);
            ready = true;
        }

        if (!ready) address = fullAddress;

        return address;
    }

    /**
      * splitList()
      * splits a comma-separated list into components
      */
    public static Vector splitList(String list) {
        Vector components = new Vector();
        if (list == null)
            return components;

        int pos = 0;
        while (pos<list.length()) {
            int next = list.indexOf(",", pos);
            if (next<0) next = list.length();
            components.addElement(list.substring(pos, next).trim());
            pos = next+1;
        }
        return components;
    }

    /** 
      * processHeader() reads from in until a line containing only the  
      * lastLine argument in read. processHeader() interprets the lines
      * as header lines and returns a Header containing these lines.
      */
    protected Header processHeader(String lastLine) throws IOException {
        Header header = new Header();
        boolean finished = false;
        while (!finished) {
            String s = in.readLine();
            finished = s.equals(lastLine);
            if (!finished) {
                int pos = s.indexOf(": ");
                if (pos>=0) {
                    header.addHeaderLine(s.substring(0,pos).trim(), s.substring(pos+1, s.length()).trim());
                }
            }
        }
        return header;
    }

    protected int readReply() throws IOException {
        replyString = in.readLine();
//        Log.out("<--"+replyString);
        try {
            String codeString = (replyString.length()>3?replyString.substring(0,3):replyString);
            reply = Integer.parseInt(codeString);
        } catch (NumberFormatException e) {
            reply = -1;
        }
        return reply;
    }

    protected static String convertFromDotLine(String s) throws IOException {
        return (s.length()>=2 && s.substring(0,2).equals("..")) 
		? s.substring(1,s.length()) 
		: s;
    }

    protected String readDotString() throws IOException {
        String reply = "";
        boolean finished=false;
        do {
            String s = in.readLine();
//            Log.out("<--"+s);
	        finished = s.equals(".");
            if (!finished) {
                reply = reply + convertFromDotLine(s) + "\n";
            }
        } while (!finished);
        if (reply.equals("")) reply = "\n";
        return reply;
    }

    protected void writeDotString(String s) throws IOException {
        BufferedReader reader = new BufferedReader(new StringReader(s));
        for (String line = reader.readLine(); line != null; line = reader.readLine() ) {
            if (line.length()>=1 && line.substring(0,1).equals(".")) line = "." + line;
            out.write(line+CRLF);
        } 
        out.write("."+CRLF);
    }
}

