/**
 * Title:	 Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author	 Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.text.*;
import java.util.*;

public class NewsClient extends NetClient {
    /**
      * This class implements RFC977, Network News Transfer Protocol
      * Implemented commands:
      * GROUP		selectGroup()
      * POST		postArticle()
      * AUTHINFO USER	login()
      * AUTHINFO PASS	login()
      * ARTICLE 	getArticle()
      * BODY		getBody()
      * HEAD		getHeader()
	  * MODE READER modeReader()
      * Unimplemented commands:
      * STAT, HELP, IHAVE, LAST, LIST, NEWGROUPS,
      * NEWNEWS, NEXT, QUIT, SLAVE
      */

    public static final int DEFAULT_PORT=119;
    protected boolean postingAllowed;

    public NewsClient() {}

    public NewsClient(String host, int port) throws IOException, UnknownHostException {
		super(host, port);
    }

    public NewsClient(String host) throws IOException, UnknownHostException {
		super(host, DEFAULT_PORT);
    }

    public void connect(String host, int port) throws IOException, UnknownHostException {
		Util.DEBUG(3, "Connecting to news server "+host+" at port "+port);
		super.connect(host, port);
		readReply();
		if (reply == 200) postingAllowed = true;
		else if (reply == 201) postingAllowed = false;
		else throw new IOException("NNTP connect failed: "+replyString);
    }

    public void connect (String host) throws IOException, UnknownHostException {
		connect(host, DEFAULT_PORT);
    }

    public void disconnect() throws IOException {
		if (sendCommand("QUIT", "") != 205) {
			throw new IOException("NNTP quit connect failed: "+replyString);
		}
		super.disconnect();
    }

    public void modeReader() throws IOException {
		int reply = sendCommand("MODE READER", "");
		if (reply == 200) {
			// OK, you can post
			Util.DEBUG(4, "Info: MODE READER returned 200, you can post");
		} else if (reply == 201) {
			// OK, but can't post
			Util.DEBUG(2, "Warning: can't post to news server");
		} else {
			// Something else
			Util.DEBUG(3, "Info: news server doesn't understand MODE READER: "+replyString);
		}
    }

    public void login(String user, String password) throws IOException {
		Util.DEBUG(3, "Authenticating...");
		int reply = sendCommand("authinfo user", user);
		if (reply == 381){ // password required
			reply = sendCommand("authinfo pass", password);
			if (reply != 281){
				throw new IOException("NNTP authinfo pass: " + getReplyString());
			}
		} else {
			throw new IOException("NNTP authinfo user: " + getReplyString());
		}
		Util.DEBUG(3, "Authentication successful");
    }

    public void selectGroup(String group, NewsgroupInfo info) throws IOException {
		Util.DEBUG(3, "Selecting newsgroup "+group);
		sendCommand("GROUP", group);
		if (reply==211) {
			try {
				int pos = replyString.indexOf(" ")+1;
				int next = replyString.indexOf(" ", pos);
				info.available = Integer.parseInt(replyString.substring(pos, next));
				pos = next+1; next = replyString.indexOf(" ", pos);
				info.first = Integer.parseInt(replyString.substring(pos, next));
				pos = next+1; next = replyString.indexOf(" ", pos);
				info.last = Integer.parseInt(replyString.substring(pos, next));
				info.name = replyString.substring(next+1, replyString.length());
			} catch (Exception e) {
				throw new IOException("Error opening "+group+": "+replyString+";"+e.getMessage());
			}
		} else
			throw new IOException("Error opening "+group+": "+replyString);
    }

    public Header getHeader(String msgid) throws IOException {
		if (sendCommand("HEAD", msgid) != 221) {
			throw new IOException("Error getting header: "+replyString);
		}
		return processHeader(".");  
    }

    public Header getHeader(int number) throws IOException {
		if (sendCommand("HEAD", Integer.toString(number)) != 221) {
			throw new IOException("Error getting header: "+replyString);
		}
		return processHeader(".");
    }

    public String getBody(int number) throws IOException {
		if (sendCommand("BODY", Integer.toString(number)) != 222) {
			throw new IOException("Error getting article body: "+replyString);
		}
		return readDotString();
    }

    public String getBody(String msgid) throws IOException {
		if (sendCommand("BODY", msgid) != 222) {
			throw new IOException("Error getting article body: "+replyString);
		}
		return readDotString();
    }

    public NewsArticle getArticle(int number) throws IOException {
		if (sendCommand("ARTICLE", Integer.toString(number)) != 220) {
			throw new IOException("Error getting article: "+replyString);
		}
		return new NewsArticle(processHeader(""), readDotString());
    }			

    public NewsArticle getArticle(String msgid) throws IOException {
		if (sendCommand("ARTICLE", msgid) != 220) {
			throw new IOException("Error getting article: "+replyString);
		}
		return new NewsArticle(processHeader(""), readDotString());
    }

    public void postArticle(Header header, String body) throws IOException {
		if (sendCommand("POST", "") != 340) {
			throw new IOException("NNTP POST: "+getReplyString());
		}
		out.write(header.toString());
		out.write(CRLF);
		writeDotString(body);
		out.flush();
		if (readReply() != 240) {
			throw new IOException("NNTP POST: "+getReplyString());
		}
	}
}
