/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class NewsRCGroup {
    protected NewsRC parent;
    protected String group;
    protected boolean subscribed;
    protected Vector ranges;

    public NewsRCGroup(NewsRC parent, String s) {
        this.parent = parent;
        ranges = new Vector();
        parseLine(s);
    }

    public NewsRCGroup(NewsRC parent, String group, boolean subscribed) {
        this.parent = parent;
        this.group = group;
        this.subscribed = subscribed;
        ranges = new Vector();
    }

    public boolean isRead(long index) {
        boolean ready = false;
        boolean found = false;
        for (int i=0; i<ranges.size() && !ready; i++) {
            Range range = (Range) ranges.elementAt(i);
            if (range.from <= index && range.to>=index) found = true;
            ready = range.from > index;
        }
        return found;   
    }

    public void markRead(long index) {
        markRead(new Range(index, index));
    }

    public void markRead(long start, long end) {
        markRead(new Range(start, end));
    }

    public void markRead(Range range) {
//        Log.AddEntry("  adding "+range.toString()+" to "+toString(), Log.TEXT, false);
        boolean ready = false;
        for (int i=0; i<ranges.size() && !ready; i++) {
            Range current = (Range) ranges.elementAt(i);
            if (current.from >= range.from) {
                ranges.insertElementAt(range, i);
                ready = true;
            }
        }
        if (!ready) {
            ranges.addElement(range);
        }
        sanitize();
        parent.markDirty();
//        Log.AddEntry("  result: "+toString(), Log.TEXT, false);
    }

    public void markUnread(long index) {
        boolean ready = false;
        for (int i=0; i<ranges.size() && !ready; i++) {
            Range range = (Range) ranges.elementAt(i);
            ready = range.from > index;
            if (range.from <= index && range.to>=index) {
                if (range.from == index) {
                    range.from++; if (range.from>range.to) ranges.removeElement(range);
                } else if (range.to == index) {
                    range.to--; if (range.from>range.to) ranges.removeElement(range);
                } else {
                    Range range2 = new Range(index+1, range.to);
                    ranges.insertElementAt(range2, i+1);
                    range.to = index-1;
                }
            }
        }
        // no sanitize() necessary
        parent.markDirty();
    }

    public String toString() {
        boolean first = true;
        String s = group + (subscribed ? ":" : "!");
        for (Enumeration e = ranges.elements(); e.hasMoreElements(); ) {
            Range range = (Range) e.nextElement();
            s = s + (first ? " " : ",") + range.toString();
            first = false;
        }
        return s;
    }

    protected void parseLine(String line) {
//        Log.AddEntry("Line: *" + line + "*", Log.TEXT, false);
        int pos = line.indexOf(":");
        if (pos == -1) pos = line.indexOf("!"); 
        if (pos != -1) {
            group = line.substring(0, pos);
            subscribed = (line.substring(pos, pos+1).equals(":") ? true : false);
            if (pos+2 <= line.length()){
                StringTokenizer magic = new StringTokenizer(line.substring(pos+2,line.length()), ",");
                while (magic.hasMoreTokens()) {
                    String t = magic.nextToken();
                    parseToken(t);
                }
            }
        }  else {
            group = "unknown";
        }  
//        Log.AddEntry("Result: *" + toString() + "*", Log.TEXT, false);
    }

    protected void parseToken(String t) {
        int pos = t.indexOf("-");
        if (pos == -1) {
            long index = Long.parseLong(t);
            markRead(index);
        } else {
            long start = Long.parseLong(t.substring(0,pos));
            long end = Long.parseLong(t.substring(pos+1,t.length()));
            markRead(start,end);
        }
    }

    protected void sanitize() {
        for (int i=0; i<ranges.size()-1; i++) {
            Range current = (Range) ranges.elementAt(i);
            Range next = (Range) ranges.elementAt(i+1);
            if (current.to >= next.from-1) {
                current.to = Math.max(current.to,next.to);
                ranges.removeElementAt(i+1);
                i--;
            }
        }
    }
  
}

