/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class PalmDatabase {
    String name; // The name of the DB on the Palm
    Vector cache; // contains cached record from the Palm
    private Class recordType; // The type of the records in this DB
    int recordCount = 0; // The number of records of the DB 
    int DB = 0;	// The Palm DB handle
    private static PalmDatabase currentlyOpen = null; // the PalmDatabase that currently has its Palm DB open

    private int currentIndex = 0;// index for firstRecord/nextRecord

    public static final int READ_AHEAD = 1;
    public static final int ALLOW_CREATE = 2;
    
    public PalmDatabase(String name, int creator, int type, Class recordType, int flags) throws IOException { 
        this.name = name;
        this.recordType = recordType;

        try {
            open();
        } catch (SyncException e) {
            if ((flags & ALLOW_CREATE) != 0) {
                create(creator, type);
            } else 
                throw e;
        }

        open();
        recordCount = SyncManager.getDBRecordCount(DB);
	Util.DEBUG(3, "Record count for "+name+": "+recordCount);

        if ((flags & READ_AHEAD) != 0) {
            try {
                fillCache(); 
            } catch (SyncException e) {
                throw new IOException("Could not read database "+name);
            }
        } else cache = new Vector();
    }

    public void create(int creator, int type) throws IOException {
        Util.DEBUG(1,"Creating database "+name);
        if (currentlyOpen != null) {
            currentlyOpen.close();
        }
        DB = SyncManager.createDB(creator, SyncManager.DB_RECORD, 0, name, type);
        currentlyOpen = this;
    }

    public void clearAllRecords() throws IOException {
        open();
        SyncManager.purgeAllRecs(DB);
        cache.removeAllElements();
        recordCount = 0;
        currentIndex = 0;
    }

    public void flushCache() {
        cache.removeAllElements();
    }

    public void flushFromCache(PalmRecord record) {
        cache.removeElement(record);
    }

    public int getRecordCount() { return recordCount; }

    public PalmRecord getRecordByID(int id) {
        PalmRecord record = null;

        // first search cache
        boolean found = false;
        for (Enumeration e = cache.elements(); e.hasMoreElements() && !found; ) {
            record = (PalmRecord) e.nextElement();
            found = (record.getId() == id);
        }

        if (!found) {
            // now get it from Palm
            try {
                open();
                record = (PalmRecord)recordType.newInstance();
                record.setId(id);
                SyncManager.readRecordById(DB, record);
                addRecordToCache(record);
            } catch (Exception e) {
                record = null;
            }
        }
        return record;
    }

    public PalmRecord getRecordByIndex(int index) {
        PalmRecord record = null;
        boolean found = false;

	Util.DEBUG(5, "Getting record #"+index+" from "+name);

        // first search cache
        int pos = findSortPosition(index);
        if (pos<cache.size()) {
            record = (PalmRecord) cache.elementAt(pos);
            found = (record.getIndex() == index); 
        }

        if (!found) {
//            Log.AddEntry("Record #"+index+" not in cache, getting from Palm", Log.TEXT, false);
            // now get it from Palm
            try {
                open();
                record = (PalmRecord)recordType.newInstance();
                record.setIndex(index);
                SyncManager.readRecordByIndex(DB, record);
                cache.insertElementAt(record, pos);
            } catch (Exception e) {
                record = null;
            }
        }
        return record;
    }

    int findSortPosition(int index) {
        int from = 0;
        int to = cache.size();
        int i = (from + to) / 2;
        boolean atEnd = false;

        boolean found = false;
        while (from < to && !found) { // && count<12) {
            i = (from + to) / 2;
            PalmRecord record = (PalmRecord) cache.elementAt(i);
	    atEnd = index>record.getIndex();
            if (index<record.getIndex()) {to = i;}
            else if (index == record.getIndex()) { found = true; }
            else from = i+1; 
        }
        if (!found && atEnd) i++;
        return i;
    }

    public PalmRecord firstRecord() {
        currentIndex = 0;
        return nextRecord();
    }

    public PalmRecord nextRecord() {
        if (currentIndex >= getRecordCount()) {
            return null;
        }

        // FIXME: make better use of the fact that cache is ordered on index

        PalmRecord record = getRecordByIndex(currentIndex);
        currentIndex++;
        return record;
    }

    protected void finalize() throws Throwable {
        close();
        super.finalize();
    }

    private void fillCache() throws IOException {
//        Log.AddEntry("Filling cache for "+name, Log.TEXT, false);

        cache = new Vector(); 
        try {
            open(); 
            for (int i=0; i<recordCount; i++) {
                PalmRecord record = (PalmRecord)recordType.newInstance();
                record.setIndex(i);
                SyncManager.readRecordByIndex(DB, record);

                cache.addElement(record);
            }
        } catch (Exception e) {
            Log.AddEntry(e.getMessage(), Log.TEXT, false);
            e.printStackTrace(); 
            throw new IOException("fillcache from "+name+": could not read records: "+e.getMessage());
        }
    }

    void addRecordToCache(PalmRecord record) throws IOException {
        // adds record to cache in proper place; record.index needs to be set
        int index = record.getIndex();
        int pos = findSortPosition(index);
        cache.insertElementAt(record, pos);
    }

    void open() throws SyncException {
        // Only one database can be open at any time!
        // This is a constraint of the HotSync protocol
        if (currentlyOpen != this) {
            if (currentlyOpen != null) {
                currentlyOpen.close();
            }
            DB = SyncManager.openDB(name, 0, 
        		SyncManager.OPEN_READ | SyncManager.OPEN_WRITE | SyncManager.OPEN_EXCLUSIVE);
            currentlyOpen = this;
        }
    }

    public void close() {
        if (currentlyOpen == this) {
            try {
                SyncManager.closeDB(DB);
            } catch (SyncException e) {
                Log.out("Error closing DB: "+e.getMessage()+". This should only happen when the previous HotSync failed.");
            }
            currentlyOpen = null;
        }
    }

    public void addRecord(PalmRecord record) throws SyncException, IOException {
        record.setId(0);
        record.setIndex(recordCount);
        open();
        SyncManager.writeRec(DB, record);
        addRecordToCache(record); // also updates cacheOkFrom, or else it remains valid
        recordCount++;
    }

    public void deleteRecord(PalmRecord record) throws SyncException, IOException {
        int index = record.getIndex();

        Util.DEBUG(2,"Deleting record "+index+" from "+name);
        
        cache.removeElement(record);
        open();
        SyncManager.deleteRecord(DB, record);

        recordCount--;

        int pos = findSortPosition(index);
        for (int i=pos; i<cache.size(); i++) {
            PalmRecord temp = (PalmRecord) cache.elementAt(i);
            temp.setIndex(temp.getIndex()-1);
        }

        if (record.getIndex() < currentIndex) currentIndex--;
    }

    /* Warning: updateRecord() is known to destroy he ordering of Palm DB records.
     */
    public void updateRecord(PalmRecord record) throws SyncException, IOException {
//        Log.AddEntry("Update record "+record.getId()+", index "+record.getIndex()+" to "+name, Log.TEXT, false);
        open();
        SyncManager.writeRec(DB, record);
    }

    public String toString() {
        String s = "Cache for "+name+" (indices):";
        for (Enumeration e = cache.elements(); e.hasMoreElements(); ) {
            PalmRecord record = (PalmRecord) e.nextElement();
            s = s + " " + record.getIndex();
        }
        return s;
    }

//    public String toFormattedString() {
//        String s = "";
//        for (Enumeration e = cache.elements(); e.hasMoreElements(); ) {
//            PalmRecord record = (PalmRecord) e.nextElement();
//            s = s + record.toFormattedString() + "\n";
//        }
//        return s;
//    }
}

    

