/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

/**
 * Notice: this class implements Yanoff 1.4 preferences. Yanoff 1.3 preferences are
 * COMPLETELY different. 
 */

import palm.conduit.*;
import java.util.*;
import java.io.*;

public class Preferences {
    // General preferences
    public int versionMajor;
    public int versionMinor;

    public int idPurgeDays;   
    public int artPurgeDays;  
    public int sampleArts;    
    public int artMaxKB;      
    public int artSepPos;     
    public int ngSepPos;      

    public boolean thread;     
    public boolean beep;       
    public boolean cutSig; 
    public boolean jumpBack;

    public int fromStyle;
    public int wordWrapLength;  

    // Poll preferences
    public int pollWhat;
    public int pollFrom;
    public int pollServer;
    public boolean postWhenPoll;

    static final short PREF_ID_MAIN = 1;
    static final short PREF_ID_POLL = 2;

    public static final int FROM_STYLE_ADDRESS = 0;
    public static final int FROM_STYLE_NAME = 1;
    public static final int FROM_STYLE_BOTH = 2;

    public static final int POLL_WHAT_ARTICLES = 0;
    public static final int POLL_WHAT_HEADERS = 1;
    public static final int POLL_WHAT_BODIES = 2;
    public static final int POLL_WHAT_SELECTED = 3;
    public static final int POLL_WHAT_COMPLETE = 4;

    public static final int POLL_FROM_ACTIVE_GROUP = 0;
    public static final int POLL_FROM_ALL_SERVERS = 1;
    public static final int POLL_FROM_ONE_SERVER = 2;

//    int creatorID;

    public void setMainDefault() {
        versionMajor = 1;
        versionMinor = 4;
        
        idPurgeDays = 3;   
        artPurgeDays = 2;  
        sampleArts= 20;    
        artMaxKB= 2;      
        artSepPos = 30;     
        ngSepPos = 30;      

        thread = false;     
        beep = true;       
        cutSig = false;
        jumpBack = true;

        fromStyle = FROM_STYLE_ADDRESS;
        wordWrapLength = 65;  
    }

    public void setPollDefault() {
        pollWhat = POLL_WHAT_ARTICLES;
        pollFrom = POLL_FROM_ALL_SERVERS;
        pollServer = 2;
        postWhenPoll = true;
    }

    private void readMainPrefs(int creatorID) throws SyncException, IOException {
//        SyncVersionException e2;
        byte[] buf = new byte[1024]; // KLUDGE: This will work, but how many bytes are required?
        short retBytes = 0;
        AppPreferenceInfo prefs = null;

        try {
            prefs = SyncManager.readAppPreference(creatorID, 
                PREF_ID_MAIN, (short)2, retBytes, true, buf);
        } catch (SyncException e) {
            if (e.errorCode == SyncException.FILE_NOT_FOUND) {
                throw new SyncException("Please configure Yanoff on Palm first");
            } else { 
                throw e;
            }
        }

        versionMajor = prefs.version >> 8;
        versionMinor = prefs.version & 0xff;

        if ( (versionMajor<1) || ( (versionMajor==1) && (versionMinor<4) ) ) {
            Log.AddEntry("Yanoff version "+versionMajor+"."+versionMinor+" not supported by this version of the Yanoff Conduit.", Log.SYNC_ABORTED, false);
            throw new SyncException("Yanoff version is "+versionMajor+"."+versionMinor+"; need v1.4");
        } 

        DataInputStream in = new DataInputStream(new ByteArrayInputStream(prefs.getBytes()));
    
        idPurgeDays = in.readShort();   
        artPurgeDays = in.readShort();  
        sampleArts = in.readShort();    
        artMaxKB = in.readShort();      
        artSepPos = in.readShort();     
        ngSepPos = in.readShort();      

        thread = in.readBoolean();     
        beep = in.readBoolean();       
        cutSig = in.readBoolean(); 
        jumpBack = in.readBoolean();
    
        fromStyle = in.readByte();  
        wordWrapLength = in.readByte();  

        in.close();
    }

    public void readPollPrefs(int creatorID) throws SyncException, IOException {
        byte[] buf = new byte[1024]; // KLUDGE: This will work, but how many bytes are required?
        short retBytes = 0;
        AppPreferenceInfo prefs = null;

        try {
            prefs = SyncManager.readAppPreference(creatorID, 
                PREF_ID_POLL, (short)2, retBytes, true, buf);
        } catch (SyncException e) {
            if (e.errorCode == SyncException.FILE_NOT_FOUND) {
                Util.DEBUG(1, "No poll preferences found, using default values");
                setPollDefault();
                return;
            } else { 
                throw e;
            }
        }
        // Maybe do version checking here? Nah.

        DataInputStream in = new DataInputStream(new ByteArrayInputStream(prefs.getBytes()));
    
        pollWhat = in.readByte();   
        pollFrom = in.readByte();  
        pollServer = in.readShort();    
        postWhenPoll = (in.readShort() == 0) ? false : true;      

        in.close();
    }
 
    public void readPrefs(int creatorID) throws SyncException, IOException {
        readMainPrefs(creatorID);
        readPollPrefs(creatorID);
    }

    private void writeMainPrefs(int creatorID) throws SyncException, IOException {
        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream(byteOut);
    
        out.writeShort(idPurgeDays);   
        out.writeShort(artPurgeDays);  
        out.writeShort(sampleArts);    
        out.writeShort(artMaxKB);      
        out.writeShort(artSepPos);     
        out.writeShort(ngSepPos);      

        out.writeBoolean(thread);     
        out.writeBoolean(beep);       
        out.writeBoolean(cutSig); 
        out.writeBoolean(jumpBack);
    
        out.writeByte(fromStyle);  
        out.writeByte(wordWrapLength);

        byte[] buf = byteOut.toByteArray();

        out.close();

        try {
            short version = (short) (versionMajor << 8 + versionMinor);
            SyncManager.writeAppPreference(version, creatorID, 
                PREF_ID_MAIN, true, buf);
        } catch (SyncException e) {
            if (e.errorCode == SyncException.FILE_NOT_FOUND) {
                throw new SyncException("Please configure Yanoff on Palm first");
            } else { 
                throw e;
            }
        }
    }

    public void writePollPrefs(int creatorID) throws SyncException, IOException {
        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
        DataOutputStream out = new DataOutputStream(byteOut);
        out.writeByte(pollWhat);   
        out.writeByte(pollFrom);  
        out.writeShort(pollServer);    
        out.writeBoolean(postWhenPoll);
        byte[] buf = byteOut.toByteArray();
        out.close();

        try {
            short version = (short) (versionMajor << 8 + versionMinor);
            SyncManager.writeAppPreference(version, creatorID, 
                PREF_ID_POLL, true, buf);
        } catch (SyncException e) {
            if (e.errorCode == SyncException.FILE_NOT_FOUND) {
                Util.DEBUG(1, "No poll preferences found");
                return;
            } else { 
                throw e;
            }
        }
    }
 
    public void writePrefs(int creatorID) throws IOException, SyncException {
        writeMainPrefs(creatorID);
        writePollPrefs(creatorID);
    }

    public String toFormattedString() {
        return ("Yanoff Preferences: {\r\n" +
                "  Version: " + versionMajor+"."+versionMinor + "\r\n" +
                "  idPurgeDays: " +  idPurgeDays + "\r\n" +
                "  artPurgeDays: " +   artPurgeDays   + "\r\n" +
                "  sampleArts: " +    sampleArts    + "\r\n" +
                "  artMaxKB: " +      artMaxKB      + "\r\n" +
                "  artSepPos: " +      artSepPos      + "\r\n" +
                "  ngSepPos: " +       ngSepPos       + "\r\n" +
                "  thread: " +      thread      + "\r\n" +
                "  beep: " +        beep        + "\r\n" +
                "  cutSig: " +  cutSig  + "\r\n" +
                "  jumpBack: " +  jumpBack  + "\r\n" +
                "  fromStyle: " +  fromStyle  + "\r\n" +
                "  wordWrapLength: " + wordWrapLength + "\r\n" +
                "  pollWhat: " + pollWhat + "\r\n" +
                "  pollFrom: " + pollFrom + "\r\n" +
                "  pollServer: " + pollServer + "\r\n" +
                "  postWhenPoll: " + postWhenPoll + "\r\n" +
                "}\r\n");
    }
}
 
