/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.text.*;
import java.util.*;

public class SMTPClient extends NetClient {
    /** 
      * This class implements RFC821, Simple Mail Transfer Protocol
      * Unimplemented commands:
      * HELO, MAIL, RCPT, DATA, SEND, SOML, SAML, RSET, 
      * VRFY, EXPN, HELP, NOOP, QUIT, TURN
      */
    public final int DEFAULT_PORT=25;

    public SMTPClient() {}

    public SMTPClient(String host, int port) throws IOException, UnknownHostException {
        super();
	connect(host, port);
    }

    public SMTPClient(String host) throws IOException, UnknownHostException {
        super();
	connect(host, DEFAULT_PORT);
    }

    public void connect(String host, int port) throws IOException, UnknownHostException {
        super.connect(host, port);
        readReply();
        if (reply != 220) throw new IOException("SMTP connect failed: "+replyString);
    }

    public void connect(String host) throws IOException, UnknownHostException {
        connect(host, DEFAULT_PORT);
    }

    public void disconnect() throws IOException {
        if (sendCommand("QUIT", "") != 221) {
            throw new IOException("NNTP quit connect failed: "+replyString);
        }
        super.disconnect();
    }

    public void login(String host) throws IOException {
        int reply = sendCommand("HELO", host);
        if (reply != 250){ // OK
            throw new IOException("SMTP HELO: " + getReplyString());
        }
    }

    public void mailMessage(Header header, String body) throws IOException {
        if (sendCommand("MAIL FROM:", "<"+extractEmailAddress(header.valueOf("from"))+">") != 250) {
            throw new IOException("SMTP MAIL FROM: "+getReplyString());
        }

        // to: field may list several recipients: addr1, addr2, addr3
        Vector recipients = splitList(header.valueOf("to"));
        for (Enumeration e=recipients.elements(); e.hasMoreElements();) {
            String recipient = (String)e.nextElement();
            recipient = extractEmailAddress(recipient);
            if (sendCommand("RCPT TO:", "<"+recipient+">") != 250) {
                Util.DEBUG(1, "Error in SMTP RCPT TO: "+getReplyString());
            }
        }

        if (sendCommand("DATA", "") != 354) {
            throw new IOException("SMTP DATA: "+getReplyString());
        }

        out.write(header.toString());
        out.write(CRLF);
        writeDotString(body);
        out.flush();
        if (readReply() != 250) {
            throw new IOException("SMTP error: "+getReplyString());
        }
    }
}

