/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import java.io.*;
import java.util.*;
import palm.conduit.*;

public class SubscriptionRecord extends PalmRecord {
    public int last;
    public int ng_flags;
    public int ng_artmaxkb;
    public int ng_downmax;
    public int ng_samplearts;
    public int server;
    public byte padding1;
    public byte padding2;
    public byte padding3;
    public boolean threadArticles;
    public int lastTop;		// the last top article when leaving the group
    public int titleContents;	// should be the subject displayed in article view or the author?
    public String dbid;
    public String newsgroup;
    public String headers;

    static final int FLAG_INTERNAL = 1;
    static final int FLAG_DONT_POLL = 2;
    static final int FLAG_POLL_NEXT = 4; // poll next n articles instead of the last ones

    static final int TITLE_SUBJECT = 0;
    static final int TITLE_AUTHOR = 1;

    public Newsgroup articles; // points to a Vector of NewsgroupRecord elements
				// was a Vector is older versions of conduit

    public void writeData(DataOutputStream out) throws java.io.IOException {
        out.writeInt(this.last);
        out.writeShort(this.ng_flags);
        out.writeShort(this.ng_artmaxkb);
        out.writeShort(this.ng_downmax);
        out.writeShort(this.ng_samplearts);
        out.writeShort(this.server);
        out.writeByte(padding1);
        out.writeByte(padding2);
        out.writeByte(padding3);
        out.writeBoolean(this.threadArticles);
        out.writeShort(this.lastTop);
        out.writeShort(this.titleContents);

        if (dbid != null) {
            out.write(dbid.getBytes());
        }
        out.writeByte(0);

        if (newsgroup != null) {
            out.write(newsgroup.getBytes());
        }
        out.write(0);

        if (headers != null) {
            out.write(headers.getBytes());
        }
        out.write(0);
    }

    public void readData(DataInputStream in) throws java.io.IOException {
        this.last = in.readInt();
        this.ng_flags = in.readShort();
        this.ng_artmaxkb = in.readShort();
        this.ng_downmax = in.readShort();
        this.ng_samplearts = in.readShort();
        this.server = in.readShort();
        padding1 = in.readByte();
        padding2 = in.readByte();
        padding3 = in.readByte();
        this.threadArticles = in.readBoolean();
        this.lastTop = in.readShort();
        this.titleContents = in.readShort();
        this.dbid = readCString(in);
        this.newsgroup = readCString(in);
        this.headers = readCString(in);
    }

    public boolean is_Internal() {
        return (ng_flags & FLAG_INTERNAL) != 0;
    }

    public boolean is_DontPoll() {
        return (ng_flags & FLAG_DONT_POLL) != 0;
    }

    public boolean okToPoll() {
        return !is_Internal() && !is_DontPoll();
    }

    public boolean isPollFromStart() {
        return (ng_flags & FLAG_POLL_NEXT) != 0;
    }

    /** Returns a string representation of the object */
    public String toString() {
        return "Subscription:" + this.newsgroup + "; dbid:"+ this.dbid+"; headers:"+this.headers+ " " + super.toString();
    }

    /** Returns a formatted string representation of the object */
    public String toFormattedString() {
        return ("Newsgroup subscription record: {\r\n" +
                "  last: " + this.last + "\r\n" +
                "  ng_flags: " + this.ng_flags + "\r\n" +
                "  ng_artmaxkb: " + this.ng_artmaxkb + "\r\n" +
                "  ng_downmax: " + this.ng_downmax + "\r\n" +
                "  ng_samplearts: " + this.ng_samplearts + "\r\n" +
                "  server: " + this.server + "\r\n" +
                "  threadArticles: " + this.threadArticles + "\r\n" +
                "  lastTop: " + this.lastTop + "\r\n" +
                "  titleContents: " + this.titleContents + "\r\n" +
                "  dbid: " + this.dbid + "\r\n" +
                "  newsgroup: " + this.newsgroup + "\r\n" +
                "  headers: " + this.headers + "\r\n" +
                "}\r\n" +
                super.toFormattedString());
    }
}
