/**
 * Title:	 Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author	 Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;
import java.text.*;

public class Threader {
    Hashtable idTable;
    Wrapper root;
    ConduitProgress progress;
    int fakeMsgidCount = 0;
    PalmDatabase articles;

    void thread(Newsgroup newsgroup, PalmDatabase articles, ConduitProgress progress) {
	this.progress = progress;
	this.articles = articles;
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 1");
	// Step 1, populate messages from newsgroup, also fill idTable
	idTable = new Hashtable();
	for(NewsgroupRecord record=(NewsgroupRecord) newsgroup.firstRecord(); record != null; record = (NewsgroupRecord) newsgroup.nextRecord()) {
	    // Step 1.A from www.jwz.org/doc/threading.html
	    ArticleRecord article = (ArticleRecord) articles.getRecordByID(record.uid);
	    Wrapper slot=insertArticle(article);
	    // Step 1.B 
	    Wrapper parent = uplinkReferences(slot);
	    // Step 1.C, set parent
	    setParent(slot, parent);
	}
	// Step 2, find the root set
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 2");
	root = findRootSet();
//	  progress.setStatus("Walking tree");
//	  walkTree(root, ""+root.msgid, false);
	// Step 3, discard idTable
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 3");
	idTable.clear();
	System.gc();
	idTable = null;
	// Step 4, prune empty containers
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 4");
	if (root.child != null) pruneEmpty(root, "");
	// Step 5, group root set by subject
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 5");
	groupBySubject();
	// Step 6, done threading
	// Step 7, sort siblings
	if (Util.debugLevel>=3) progress.setStatus("Threading, step 7");
	sortChildren(root);
    }

    void cleanup(Wrapper parent) {
	Wrapper current;
	for (current=parent.child; current != null; current = current.next) {
	    if (current.child != null) cleanup(current);
	    if (current.article != null) {
		articles.flushFromCache(current.article);
		current.article = null;
	    }
	}
    }

    public void cleanup() {
//	  cleanup(root); // remove articles in threads from articles database
	root = null;
	System.gc();
    }

    Wrapper insertArticle(ArticleRecord article) {
	Wrapper slot=null;    
	String msgid = article.msgid;
	if (msgid == null) {
	    Log.out("Article without msgid: "+article.toFormattedString());
	    msgid = "Yf" + fakeMsgidCount+"$"+new Random().nextLong();
	    Log.out("Created fake msgid: "+msgid);
	}
	if (msgid != null)
	    slot = (Wrapper) idTable.get(msgid);
	if (slot != null) {
	    if (slot.article != null) {
		// Damn! another article with the same msgid exists.
		// This should not happen
		// Fake a msgid, only for threading
		msgid = msgid.substring(0,1)+fakeMsgidCount+"$"+msgid.substring(1, article.msgid.length());
		fakeMsgidCount++;
	    } else {
		slot.fillFrom(article);
	    }
	}
	if (slot == null) {
	    slot = new Wrapper();
	    slot.fillFrom(article);
	    idTable.put(msgid, slot);
	}
	return slot;
    }

    Wrapper uplinkReferences(Wrapper slot) {
	Wrapper parent = null;
	Object old_reference = null;
	if (slot.ref != null) {
	    for (Enumeration e=slot.ref.elements(); e.hasMoreElements();) {
		Object reference=e.nextElement();
		Wrapper referenced=(Wrapper) idTable.get(reference);
		if (referenced==null){
		    referenced = new Wrapper();
		    idTable.put(reference, referenced);
		}
		if (parent != null &&			// don't do first reference
			referenced.parent == null &&	    // don't screw up earlier parents
			parent != referenced && 	    // watch out for cycles
			!parent.reachable(referenced)) {       // watch out for cycles
		    referenced.parent = parent;
		    referenced.next = parent.child;
		    parent.child = referenced;
		}
		parent = referenced;
		old_reference = reference;
	    }
	}

	return parent;
    }

    void setParent(Wrapper slot, Wrapper parent) {
	 // sanity check
	 if (parent != null && (parent == slot || slot.reachable(parent))) {
	     parent = null;
	 }

	 // first unlink from current parent
	 if (slot.parent != null) { 
	     slot.parent.unlinkChild(slot);
	 }

	 // then link in slot under parent
	 if (parent != null) {
	     slot.parent = parent;
	     slot.next = parent.child;
	     parent.child = slot;
	 }
    }

    Wrapper findRootSet() {
	 Wrapper root = new Wrapper();
	 for (Enumeration e = idTable.elements() ; e.hasMoreElements() ;) {
	     Wrapper slot = (Wrapper) e.nextElement();
	     if (slot.parent == null) { 
		 if (slot.next != null) { // This should never happen, orphans don't have siblings
		     throw new Error("Error finding root set, please report to author");
		 }
		 slot.parent = root;
		 slot.next = root.child;
		 root.child = slot;
	     }
	 }
	 return root;
    }

    void walkTree(Wrapper parent, String history, boolean showall) {
	int childno;
	Wrapper current;
	for (current=parent.child, childno=1; current != null; current = current.next, childno++) {
	    if (current.msgid != null && current.msgid.equals("<957100975.10087.0.pluto.d4ee712d@news.demon.nl>")) showall=true;
	    if (Util.debugLevel >= 4) {
		progress.setCurrentNewsgroupName(history+current.msgid);
		if (showall) Log.out("Child #"+childno+" " + current.msgid + "->" + current.ref);
	    }
	    if (current.child != null) walkTree(current, history+current.msgid+",", showall);
	    if (childno > 8000) throw new Error("Cycle in tree!"+history+current.msgid+" "+current.subject);
	    if (showall && childno > 25) throw new Error("Cycle in relevant tree!"+history+current.msgid+" "+current.subject);
	}
    }

    void pruneEmpty(Wrapper parent, String history) {
	Wrapper previous,current,next;
	int childno = 0;
	for(previous=null,current=parent.child,next=current.next; current != null; previous=current,current=next,next=(current==null?null:current.next) ) {
	    childno++;
	    // Step 4.A, nuke empty containers with no children
	    if (current.article == null && current.child==null) {
	       if (previous==null) {
		   parent.child = current.next;
	       } else {
		   previous.next = current.next;
	       }
	       current=previous; // to keep the for loop sane
	    } else if (current.article == null && current.child != null &&
			   (current.parent != null || current.child.next == null)) { 
			     // Step 4.B, promote children of empty containers
		Wrapper kid;
		for (kid=current.child; kid.next != null; kid = kid.next) {
		    kid.parent = current.parent;
		}
		kid.parent = current.parent;
		kid.next = current.next;

		if (previous == null) {
		    parent.child = current.child;
		} else {
		    previous.next = current.child;
		}
		next = current.child;
		current = previous; // keep for loop sane

	    } else if (current.child != null) {
		pruneEmpty(current, history+childno+","); // Walk down the tree 
	    }
	}
    }

    void groupBySubject() {
	// Step A, create subjectTable
	Hashtable subjectTable = new Hashtable();

	// Step B, find subject of sub-trees
	for (Wrapper current=root.child; current != null; current=current.next) {
	    String subject;
	    if (current.article != null) 
		subject = current.cleanSubject;
	    else if (current.child != null && current.child.article != null) 
		subject = current.child.cleanSubject;
	    else {
		subject = null;
		Log.out("Error grouping by subject, please report to author");
	    }

	    if (subject == null || subject.equals(""))
		continue; // Give up
 
	    Wrapper previous = (Wrapper) subjectTable.get(subject);
	    if (    previous == null ||    // new subject
		    ( current.article == null && previous.article != null) ||  // rather keep empty container
		    ( !current.isReply && previous.isReply ) )	// rather keep original rather then reply
		subjectTable.put(subject, current);
	}

	// Step C, merge trees with equal cleaned subjects
	Wrapper previous=null;
	for (Wrapper current=root.child; current != null; previous=current, current=current.next) {
	    String subject = null;
	    if (current.article == null && current.child != null) 
		subject = current.child.cleanSubject;
	    else
		subject = current.cleanSubject;

	    if (subject == null || subject.equals("")) 
		continue;

	    Wrapper other = (Wrapper) subjectTable.get(subject);
	    if (other==null || other==current)	 // other==null really shouldn't happen
		continue;

	    // First unlink current
// done later
//	      if (previous == null)
//		  root.child = current.next;
//	      else
//		  previous.child = current.next;
//	      current.next = null;

	    if (current.article==null && other.article==null) {
		// If both are dummies, merge the children
		Wrapper tail;
		root.unlinkChild(current);
		for (tail = other.child; tail != null && tail.next != null; tail=tail.next)
		    ;
		tail.next = current.child;
		for (tail = current.child; tail != null && tail.next != null; tail=tail.next)
		    tail.parent = other;
		current.child = null;

		current = previous; // keep for loop happy
//	      } else if (current.article == null || (other.article != null && other.isReply && !current.isReply)) {
// Skipped because this causes other not to be a root node anymore
		// If one container is a empty and the other is not, make the non-empty 
		// one be a child of the empty, and a sibling of the other ``real''
		// messages with the same subject (the empty's children.) 

		// current is empty, make other a child of current
//		  root.unlinkChild(other);
//		  other.parent = current;
//		  other.next = current.child;
//		  current.child = other;
	    } else if (other.article == null || (current.article != null && !other.isReply && current.isReply)) {
		// other is empty, make current a child of other
		root.unlinkChild(current);
		current.parent = other;
		current.next = other.child;
		other.child = current;
		current = previous; // keep for loop happy
	    } else {
		// otherwise, make a new empty container and make both be a child of it.
		// Note: newSlot and current become a child of other
		root.unlinkChild(current);

		Wrapper newSlot = new Wrapper();
		newSlot.fillFrom(other.article);
		other.article = null;

		newSlot.next = current;
		newSlot.parent = other; 
		newSlot.child = other.child;

		for (Wrapper kid = newSlot.child; kid != null; kid=kid.next)
		    kid.parent = newSlot;
		other.child = newSlot;

		current.parent = other;
		current.next = null;
		
		current = previous; // keep for loop happy
	    }
	}
	    
	subjectTable.clear();
	subjectTable = null;
    }

    void sortChildren(Wrapper parent) {
	if (parent.child == null)
	    return;

//	  if (Util.debugLevel>=4) progress.setCurrentNewsgroupName("Sorting "+parent.subject);

	// Use the Collections framework. Put the children in a list and sort().
	// I' like to do that, but the CDKJ only support JDK1.1, and we need
	// JDK1.2 for Collections...
	// return to bubble sort
	// bubble sort seems too slow when we've got 2000 articles in a group.
	// implement Sorter class and use quicksort.

	Wrapper previous, current;
	Vector children = new Vector();

	for (current=parent.child; current != null; current = current.next) {
	    children.addElement(current);
	}
	Sorter.quicksort(children);

	int i;
	for (i=0,previous=null; i<children.size(); i++,previous=current) {
	    current=(Wrapper) children.elementAt(i);
	    if (previous == null) {
		parent.child = current;
	    } else {
		previous.next = current;
	    }
	}
	current.next = null;

	// Now recursively sort all children
	for (current=parent.child; current!=null; current=current.next) 
	    sortChildren(current);
    }

    public String toString() { 
	return root.toString(0);
    }

    public Enumeration order() {
	return new ThreadEnumeration(root);
    }

}

class Wrapper implements Comparable {
    ArticleRecord article= null;
    Wrapper parent;
    Wrapper child;  // first child
    Wrapper next;   // next element in sibling list, or null
    String subject;
    String cleanSubject;
    boolean isReply;
    Date date;
    Object msgid;
    Vector ref;

    public int compareTo(Object o) {
	if (o == null || o.getClass() != this.getClass())
	    throw new ClassCastException();
	Wrapper other = (Wrapper) o;
	if (date == null && other.date == null) return 0;
	else if (date == null) return -1;
	else if (other.date == null) return 1;
	else if (date.equals(other.date)) return 0;
	else return (date.before(other.date) ? -1 : 1);
    }

    boolean reachable(Wrapper target) {
	boolean found=false;
	for (Wrapper kid=child; kid!=null && !found; kid=kid.next) {
	    found = (kid==target) || kid.reachable(target);
	}
	return found;
    }

    void unlinkChild(Wrapper target) {
	if (child == target) {
	    child = child.next;
	    target.next = null;
	    target.parent = null;
	} else {
	    boolean found=false;
	    Wrapper previous=child;
	    for (Wrapper current=child.next; current!=null; previous=current,current=current.next) {
		found = (current==target); 
		if (found) {
		    previous.next = target.next;
			target.next = null;
			target.parent = null;
			break;
		    }
		}
		if (!found) throw new Error("Error in threading, please report to author");
	    }
	}

	void fillFrom(ArticleRecord article) {
	    this.article = article;
	    this.subject = article.subject;
	    cleanSubject();
	    if (article.date == null || article.date.equals(""))
		this.date = new Date(0);
	    else 
		try {
		    this.date = new Date(article.date);
		} catch (IllegalArgumentException e) {
		    Log.out("Cannot parse date "+article.date+" : "+e.getMessage());
		    this.date = new Date(0);
		}
//		  try {
//		      this.date = DateFormat.getDateTimeInstance(DateFormat.LONG, DateFormat.LONG).parse(article.date);
//		  } catch (ParseException e) {
//		      Log.out("Cannot parse date "+article.date+" : "+e.getMessage());
//		      this.date = new Date(0);
//		  }
//	      if (Util.debugLevel>=3) Log.out("Date parse: "+article.date+" becomes "+this.date.toString());
	    this.msgid = article.msgid;
	    this.ref = article.ref == null ? null : parseRefs(article.ref);
	}

    static Vector parseRefs(String ref) {
	Vector vector = new Vector();
	int count = 0;	      
	for (int i=0; i<ref.length() && count<100; count++) {
	    int pos1 = ref.indexOf("<", i);
	    if (pos1<0) pos1 = 0;
	    int pos2 = ref.indexOf(">", pos1)+1;
	    if (pos2<=pos1) pos2 = ref.length();
//	      if (Util.debugLevel>=4) Log.out("pos1,2 = "+pos1+","+pos2+"; msgid = *"+ref.substring(pos1,pos2)+"*");
	    vector.addElement(ref.substring(pos1, pos2));
	    i = pos2;
	}
	if (count>=100) Log.out("DEBUG: Possible error parsing "+ref);
	return vector;
    }

	void cleanSubject() {
	    // FIXME: do this better
	    cleanSubject = subject;
	    if (subject != null) {
		int pos = cleanSubject.toLowerCase().indexOf("re:");
		isReply = (pos>=0);
		if (isReply) {
		    cleanSubject = subject.substring(3, subject.length());
		}
		cleanSubject.trim();
	    }
	}

    public String toString() {
	return (String)msgid + subject + date;
    }

    public String toString(int level) {
	String s = "";
	int newLevel = level;
	if (article != null) {
	    s = s + toString()+"\n";
	    if (level>=1) s = "|-" + s;
	    for(int i=0; i<level-1; i++) s = "	" + s;
	    newLevel++;
	}
	for (Wrapper current=child; current != null; current=current.next) {
	    s = s + current.toString(newLevel);
	}
	return s;
    }

}

class ThreadEnumeration implements Enumeration {
    Wrapper current;

    ThreadEnumeration(Wrapper root) {
	current = root;
	while (current != null && current.article == null) {
	    nextStep();
	}
    }

    public boolean hasMoreElements() {
	return (current != null);
    }

    public Object nextElement() {
	Wrapper save=current;
	findNext();
	return save.article;
    }

    void nextStep() {
	if (current != null) {
	    if (current.child != null) 
		current = current.child;	  // down
	    else if (current.next != null)
		current = current.next; 	  // right
	    else {				  // up and right
		while (current != null && current.next == null)
		    current = current.parent;
		if (current != null)
		    current = current.next;
	    }
	}
    }

    void findNext() {
	do {
	    nextStep();
	} while (current != null && current.article == null);
    }
}


