/**
 * Title:	 Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:	 Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author	 Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class Yanoff {
    static final String ARTICLES_DB = "NewsArts";
    static final String SUBSCRIPTIONS_DB = "NewsSubscriptions";
    static final String SERVERS_DB = "NewsServers";
    static final String NEWSGROUP_DB_PREFIX = "NewsGroup-";

    static final String ARTICLE_CLASS = "org.vanbest.yanoff.ArticleRecord";
    static final String SUBSCRIPTION_CLASS = "org.vanbest.yanoff.SubscriptionRecord";
    static final String SERVER_CLASS = "org.vanbest.yanoff.ServerRecord";
    static final String NEWSGROUP_CLASS = "org.vanbest.yanoff.NewsgroupRecord";

    static final String OUTGOING = "!! Outgoing";

    static final String TRUNC_STRING = "\n*TRUNCATED*";
    static final String SIG_CUT_STRING = "\n-- \n";

    static final int CREATOR_ID = 0x4d4a7966;
    static final int DB_TYPE = 0x4d4a7966;

    Preferences prefs = new Preferences();
    ConduitProgress progress;
    ConduitConfigure config;
    PalmDatabase articles, subscriptions, servers;
    History history;
    NewsClient NNTP;
    SMTPClient SMTP;
    int memRequired; // the amount of memory still needed on the Palm for newsgroups updates

    public void Yanoff() {
    }

    public void open(ConduitConfigure config) throws ClassNotFoundException, InstantiationException, IllegalAccessException, IOException, SyncException, Exception  {
	this.config = config;

	prefs.readPrefs(CREATOR_ID);
	memRequired = 0;

	Util.DEBUG(1,"Yanoff version: "+prefs.versionMajor+"."+prefs.versionMinor);
	Util.DEBUG(1,"Java version: "+System.getProperty("java.vendor")+" "+System.getProperty("java.version"));
	Util.DEBUG(1,"Free memory: "+memFree()+" bytes");
	Util.DEBUG(0,"Conduit debug level: "+config.debugLevel);
	Util.DEBUG(2, "Conduit dated "+Version.BUILD_DATE);
	progress = new ConduitProgress(config);
	progress.createDialog();
	progress.setStatus("");

	readDatabases();

	int i = 0;
	for(ServerRecord server=(ServerRecord) servers.firstRecord(); server != null; server = (ServerRecord) servers.nextRecord()) {
	    if (server.serverType == ServerRecord.NEWS_SERVER) {
		String newName = server.name + " (" + server.address + ")";
		// set useNewsrc to false if server name changed.
		if ( (config.server[i]==null) || !newName.equals(config.server[i]) ) {
		    Util.DEBUG(1, "Server "+config.server[i]+" changed to "+newName+"; resetting use newsrc");
		    config.useNewsrc[i] = false;
		}
		config.server[i] = newName;
		i++;
	    }
	}
	config.numServers = i;
	progress.setNumServers(i+1); // zero counts too

	if ( (config.debugLevel & ConduitConfigure.DEBUG_DUMPALL) != 0) dumpAll();
    }

    public void close() throws IllegalAccessException, InstantiationException, IOException, SyncException {
	syncDatabases();
	progress.close();
    }

    public void pollAndPost() throws IllegalAccessException, InstantiationException, IOException, SyncException {
	int polled = 0;
	int posted = 0;
	int mailed = 0;

	if (!config.usePollPrefs || prefs.postWhenPoll) {
	    ServerRecord server=(ServerRecord) servers.getRecordByIndex(0);
	    progress.setCurrentServer(0);
	    progress.setCurrentServerName(server.address);
	    Util.DEBUG(1, "Mail server: " + server.address);
	    if (availableArticlesForServer(server.getIndex()) > 0) {
		try {
		    progress.setStatus("Connecting to server");
		    SMTP = new SMTPClient(server.address, server.port); 
		    SMTP.login(server.hostname);
		    progress.setStatus("Mailing articles");
		    mailed += mailOutgoingArticles(server.getIndex());
		    SMTP.disconnect();
		} catch (Exception e1) {
		     Log.AddEntry("An error occured: "+e1.getMessage(), Log.TEXT, false);
		     e1.printStackTrace();
		}
	    }
	}

	int first = 1;
	int last = servers.getRecordCount()-1;

	if (config.usePollPrefs) {
	    switch(prefs.pollFrom) {
	    case Preferences.POLL_FROM_ALL_SERVERS:
		first = 1;
		last = servers.getRecordCount()-1;
		break;
	    case Preferences.POLL_FROM_ONE_SERVER:
		first = prefs.pollServer;
		last = prefs.pollServer;
	    case Preferences.POLL_FROM_ACTIVE_GROUP: // We don't now what the active group is, so skip
	    default:
		first = 1;
		last = 0;
		break;
	    }
	}

	for(int i=first; i<=last; i++) {
	    ServerRecord server=(ServerRecord) servers.getRecordByIndex(i);
	    progress.setCurrentServer(i);
	    progress.setCurrentServerName(server.address);
	    Util.DEBUG(1, "News server: " + server.address + " port " + server.port);

	    if (server.getPoll()) {
		try {
		    progress.setStatus("Connecting to server");
		    NNTP = new NewsClient(server.address, server.port);
		    if (config.useModeReader) {
			NNTP.modeReader();
		    }
		    if (server.use_auth) {
			NNTP.login(server.auth_user, server.auth_pass);
			Util.DEBUG(1,"authenticated");
		    }
		    if (!config.usePollPrefs || prefs.postWhenPoll) {
			progress.setStatus("Posting articles");
			posted += postOutgoingArticlesToServer(server.getIndex());
		    }
		    progress.setStatus("Polling articles");
		    polled += pollNewsGroupsFromServer(server.getIndex());
		    NNTP.disconnect();
		    progress.setStatus("");
		} catch (Exception e1) {
		    Log.AddEntry("An error occured: "+e1.getMessage(), Log.TEXT, false);
		    e1.printStackTrace();
		}
	    } else {
		if ( (!config.usePollPrefs || prefs.postWhenPoll) &&
		     (availableArticlesForServer(server.getIndex()) > 0) ) {
		    try {
			progress.setStatus("Connecting to server");
			NNTP = new NewsClient(server.address, server.port);
			if (config.useModeReader) {
			    NNTP.modeReader();
			}
			if (server.use_auth) {
			    NNTP.login(server.auth_user, server.auth_pass);
			    Util.DEBUG(1,"authenticated");
			}
			progress.setStatus("Posting articles");
			posted += postOutgoingArticlesToServer(server.getIndex());
			NNTP.disconnect();
		    } catch (Exception e1) {
			Log.AddEntry("An error occured: "+e1.getMessage(), Log.TEXT, false);
			e1.printStackTrace();
		    }
		}
	    }
	}
	progress.setCurrentServer(0);
	progress.setCurrentServerName("");

	Util.DEBUG(1, "Polled " + polled + "; Posted " + posted + "; Mailed " + mailed);
	config.lastSync = new Date(System.currentTimeMillis());
    }

    public int availableArticlesForServer(int serverNum) {
	int availableArticles = 0;

	try {
	    Newsgroup outgoing = getNewsgroupByName(OUTGOING);

	    for(NewsgroupRecord record=(NewsgroupRecord) outgoing.firstRecord(); record != null; record = (NewsgroupRecord) outgoing.nextRecord()) {
		ArticleRecord article = (ArticleRecord) articles.getRecordByID(record.uid);
		if ( (article != null) && (article.server == serverNum))
		    availableArticles++;
	    }
	} catch (Exception e) {
	    Log.AddEntry("An error occured when trying to determine the number of "+
		 "articles to send to server #"+serverNum, Log.TEXT, false);
	    e.printStackTrace();
	} // return 0 if an error occurs

	return availableArticles;
    }

    // News server should be connected before calling this method
    public int postOutgoingArticlesToServer(int serverNum) throws IllegalAccessException, InstantiationException, IOException, SyncException {
	int postedArticles = 0;

	Newsgroup outgoing = getNewsgroupByName(OUTGOING);

	for(NewsgroupRecord record=(NewsgroupRecord) outgoing.firstRecord(); record != null; record = (NewsgroupRecord) outgoing.nextRecord()) {
	    ArticleRecord article = (ArticleRecord) articles.getRecordByID(record.uid);
	    if ( (article != null) && (article.server == serverNum)) {
		Util.DEBUG(2,"Posting article " + article.msgid);
		if (postArticle(article)) {
		    postedArticles++;
		    articles.deleteRecord(article);
		    outgoing.deleteRecord(record);
		}
	    }
	}

	return postedArticles;
    }

    public int pollNewsGroupsFromServer(int serverNum) throws IllegalAccessException, InstantiationException, IOException, SyncException {
	int numberPolled = 0;
	NewsRC newsRC = null;
	NewsRCGroup newsRCGroup = null;

	if (config.useNewsrc[serverNum-1]) newsRC = new NewsRC(config.newsrcPath[serverNum-1]);

	for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
	    if (subscription.server == serverNum && subscription.okToPoll()) {
		progress.setCurrentNewsgroupName(subscription.newsgroup);
		if (newsRC != null) newsRCGroup = newsRC.getGroup(subscription.newsgroup);
		if (!config.usePollPrefs || prefs.pollWhat==Preferences.POLL_WHAT_ARTICLES ||
			prefs.pollWhat==Preferences.POLL_WHAT_HEADERS) {
		    numberPolled += pollNewNews(subscription, newsRCGroup);
		} else {
		    numberPolled += pollBodies(subscription);
		}
		if (newsRCGroup != null)
		    Util.DEBUG(3,"RC line for " + subscription.newsgroup + ": " + newsRCGroup.toString());
	    }
	}
	progress.setCurrentNewsgroupName("");

	if (newsRC != null) newsRC.save();
	return numberPolled;
    }

    int pollNewNews(SubscriptionRecord subscription, NewsRCGroup newsRCGroup) throws IllegalAccessException, InstantiationException, IOException, SyncException {
	int numberPolled = 0;
	int maxToPoll = 0;
	NewsgroupInfo info = new NewsgroupInfo();

//	  Log.out(subscription.toFormattedString());

	NNTP.selectGroup(subscription.newsgroup, info);

	int from = info.first;
	int to = info.last;

	boolean fromStart;
	if (subscription.last == 0) { // freshly subscribed newsgroup
	    maxToPoll = subscription.ng_samplearts;
	    fromStart = false;
	} else {
	    maxToPoll = subscription.ng_downmax;
	    fromStart = subscription.isPollFromStart();
	}
	from = Math.max(from, subscription.last+1); 

	numberPolled += pollArticles(subscription, newsRCGroup, from,to, fromStart, maxToPoll);

	return numberPolled;
    }

    int pollArticles(SubscriptionRecord subscription, NewsRCGroup newsRCGroup, int from, int to, boolean fromStart, int maxToPoll) {
	int numberPolled = 0;
	int start = from;

	// Make sure we get the last maxtoPoll articles if asked for
	// FIXME: maybe we get fewer articles because some have expired 
	// or are marked read in newsrc
	if ((!fromStart) && (to - from + 1 > maxToPoll)) {
	    from = to - maxToPoll + 1;
	}

	int maxArticles = maxToPoll;
	if (to-from+1 < maxToPoll) maxArticles = to-from+1;
	if (maxArticles < 0) maxArticles = 0;
	progress.setNumArticles(maxArticles);
	progress.setCurrentArticle(0);

	for (int i=from; (i<=to) && (numberPolled < maxToPoll); i++) {
	    progress.setCurrentArticle(numberPolled);
	    NewsgroupRecord newsgroupRecord = null;
//	      Log.AddEntry("Article #" + i + " in " + subscription.newsgroup + ": " + newsRCGroup.isRead(i), Log.TEXT, false);
	    if( (newsRCGroup==null) || !newsRCGroup.isRead(i)) {
		ArticleRecord article = retrieveArticle(i);
		if (article != null && !history.msgidExists(article.msgid)) {
		    int saveLast = subscription.last;
		    Util.DEBUG(4, "Got article: "+article.msgid);
		    tidyUpArticle(article, subscription.ng_artmaxkb*1024);
		    // FIXME: make smaller try-catch blocks in order to correctly 
		    // handle errors in every action.
		    try {
			articles.addRecord(article);
			newsgroupRecord = new NewsgroupRecord(article.getId());
			subscription.articles.addRecord(newsgroupRecord);
			memRequired += NewsgroupRecord.SIZE_ON_PALM;
			numberPolled++;
			subscription.last = i;
			subscriptions.updateRecord(subscription);
			if (newsRCGroup != null) newsRCGroup.markRead(i);
			if (article.msgid != null) history.addMsgid(article.msgid);
			// FIXME: also add mem requirements for history here
		    } catch (IOException e) {
			Log.AddEntry("Unable to write article to handheld. Probably memory full.", Log.TEXT, false);
			break;
		    }
		    if (memRequired > memFree()) {
			Log.AddEntry("Memory full! Removing latest article", Log.TEXT, false);
			// clean up
			try {
			    articles.deleteRecord(article);
			} catch (IOException e) { }
			try {
			    if (newsgroupRecord != null) subscription.articles.deleteRecord(newsgroupRecord);
			} catch (IOException e) { }
			memRequired -= NewsgroupRecord.SIZE_ON_PALM;
			numberPolled--;
			subscription.last = saveLast;
			try {
			    subscriptions.updateRecord(subscription);
			} catch (IOException e) { }
			if (newsRCGroup != null) newsRCGroup.markUnread(i);
			try {
			    history.removeMsgid(article.msgid);
			} catch (IOException e) { }
		    break;
		    }
		}
		progress.setCurrentArticle(numberPolled);
	    }
	}
	progress.setNumArticles(0);
	progress.setCurrentArticle(0);
	return numberPolled;
    }

    public ArticleRecord retrieveArticle(int articleNumber) {
	ArticleRecord article = null; 
	try {
	    article = new ArticleRecord();
	    if (!config.usePollPrefs || prefs.pollWhat == Preferences.POLL_WHAT_ARTICLES) {
		NewsArticle art = NNTP.getArticle(articleNumber);
		article.body = art.body;
		fillInArticleHeader(art.header, article);
	    } else {
		Header header = NNTP.getHeader(articleNumber);
		article.body = null;
		fillInArticleHeader(header, article); 
	    }
		
	    article.rcvDate = new Date(System.currentTimeMillis());
	    article.xPostings = 1;
	    article.flags = 0;
	} catch (IOException e) {
	    Log.AddEntry("Unable to retrieve article " + articleNumber + " from server", Log.TEXT, false);
	    article = null;
	}  
	return article;
    }

    void fillInArticleHeader(Header header, ArticleRecord article) throws IOException {
	article.from = header.valueOf("from");
	article.subject = header.valueOf("subject");
	article.date = header.valueOf("date");
	article.newsgroups = header.valueOf("newsgroups");
	article.msgid = header.valueOf("message-id");
	article.ref = header.valueOf("references");
	article.fup_to = header.valueOf("followup-to");
	// XXX Maybe shove extra headers into the headers field; probably takes too much space
    }

    void tidyUpArticle(ArticleRecord article, int maxBodySize) {
	if (article.body == null) return;

	// cutoff signature if requested
	if (prefs.cutSig) { // FIXME: get cutsig flag from subscription???
	    int pos = article.body.lastIndexOf(SIG_CUT_STRING);
	    if (pos != -1) article.body = article.body.substring(0, pos);
	}

	// truncate long article
	if (article.body.length() > maxBodySize)
	    article.body = article.body.substring(0, maxBodySize) + TRUNC_STRING;
    }

    int pollBodies(SubscriptionRecord subscription) throws IllegalAccessException, InstantiationException, IOException, SyncException {
	Util.DEBUG(2, "Polling bodies for "+subscription.newsgroup);
	int numberPolled = 0;

	progress.setNumArticles(subscription.articles.getRecordCount());
	progress.setCurrentArticle(0);

	NewsgroupInfo info = new NewsgroupInfo();
	NNTP.selectGroup(subscription.newsgroup, info);

	for (int i=0; i<subscription.articles.getRecordCount(); i++) {
	    progress.setCurrentArticle(i);
	    NewsgroupRecord record=(NewsgroupRecord) subscription.articles.getRecordByIndex(i);
	    ArticleRecord article = (ArticleRecord) articles.getRecordByID(record.uid);
	    if ( ((prefs.pollWhat!=Preferences.POLL_WHAT_SELECTED) && (prefs.pollWhat != Preferences.POLL_WHAT_COMPLETE))
	      || ((prefs.pollWhat==Preferences.POLL_WHAT_SELECTED) && article.isMarked())
	      || ((prefs.pollWhat==Preferences.POLL_WHAT_COMPLETE) && article.isMarked()) ) {
		Util.DEBUG(3, "Getting body for "+article.msgid);
		if (pollArticleBody(subscription, article)) numberPolled++;
	    }
	}
	progress.setNumArticles(0);
	progress.setCurrentArticle(0);
	return numberPolled;
    }

    boolean pollArticleBody(SubscriptionRecord subscription, ArticleRecord article) {
	boolean result = false;
	if (article.body != null && !article.body.equals("") &&
		prefs.pollWhat != Preferences.POLL_WHAT_COMPLETE) {
	    Util.DEBUG(5, "One");
	    return false;
	} else {
	    Util.DEBUG(5, "Two");
	    try {
		article.body = NNTP.getBody(article.msgid);
		Util.DEBUG(5, "Three");
	    } catch (IOException e) {
		Util.DEBUG(5, "Four");
		Log.out("Unable to retrieve article body " + article.msgid + " from server");
		return false;
	    }  
	}

	Util.DEBUG(5, "Five");
	try {
	    article.rcvDate = new Date(System.currentTimeMillis());
	    if (prefs.pollWhat == Preferences.POLL_WHAT_COMPLETE) {
		if (article.body.length() > 32*1024)
		   article.body = article.body.substring(0, 32*1024) + TRUNC_STRING;
	    } else {
		tidyUpArticle(article, subscription.ng_artmaxkb*1024);
	    }
	    Util.DEBUG(5, "Six");
	    articles.updateRecord(article);
	    result = true;
	    Util.DEBUG(3, "Got body for article "+article.msgid);
	} catch (IOException e) {
	    Log.AddEntry("Unable to write article to handheld. Probably memory full.", Log.TEXT, false);
	    return false;
	}
	return result;
    }

    Newsgroup getNewsgroupByName(String ngName) {
	Newsgroup db = null;
 
	for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
	    if (subscription != null && subscription.newsgroup.equals(ngName)) {
		db = subscription.articles;
	    }
	}
	return db;
    }

    void readDatabases() throws ClassNotFoundException, InstantiationException, IllegalAccessException, IOException {
	progress.setStatus("Opening Palm databases");
	servers = new PalmDatabase(SERVERS_DB, CREATOR_ID, DB_TYPE, Class.forName(SERVER_CLASS), PalmDatabase.READ_AHEAD);
	subscriptions = new PalmDatabase(SUBSCRIPTIONS_DB, CREATOR_ID, DB_TYPE, Class.forName(SUBSCRIPTION_CLASS), PalmDatabase.READ_AHEAD);
	articles = new PalmDatabase(ARTICLES_DB, CREATOR_ID, DB_TYPE, Class.forName(ARTICLE_CLASS), 0);
	history = new History(CREATOR_ID, DB_TYPE, PalmDatabase.ALLOW_CREATE);

	for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
	    Newsgroup newsgroup = new Newsgroup(NEWSGROUP_DB_PREFIX+subscription.dbid, CREATOR_ID, DB_TYPE, 
		Class.forName(NEWSGROUP_CLASS), 0, articles);
	    subscription.articles = newsgroup;
//	      Util.DEBUG(3, subscription.newsgroup+": thread = "+subscription.threadArticles);
	}

	Util.DEBUG(1, "  " + servers.getRecordCount()+ " servers known.");
	Util.DEBUG(1, "  " + subscriptions.getRecordCount()+ " newsgroups subscribed to.");
	Util.DEBUG(2, "  " + articles.getRecordCount()+ " articles in database.");
	Util.DEBUG(2, "  " + history.getRecordCount()+ " message ID's in history.");
	progress.setStatus("");
    }

    void syncDatabases() throws InstantiationException, IllegalAccessException, IOException {
	progress.setStatus("Threading");
	for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
	    if (config.allowThreading && subscription.threadArticles) {
		progress.setCurrentNewsgroupName(subscription.newsgroup);
		subscription.articles.thread(progress);
		if (articles.getRecordCount()>1000) {
		    // This is a terrible hack. The HotSync JSync sets the Java VM size to
		    // 16 M. If we read too many articles (thousands), the Java VM will run full.
		    // So, we just clear the articles cache when needed...
		    articles.flushCache();
		}
		progress.setCurrentNewsgroupName("");
	    }
	    subscription.articles.flushCache();
	    subscription.articles.close();
	    System.gc();
	}
	progress.setCurrentNewsgroupName("");
	subscriptions.flushCache();
	subscriptions.close();
	articles.flushCache();
	articles.close();
	servers.flushCache();
	servers.close();

	progress.setStatus("Sorting History database");
	// sort, ?purge, purge-days
	history.cleanup(true, (config.purgeMsgidDays != ConduitConfigure.NO_PURGE), config.purgeMsgidDays);
	history.flushCache();
	history.close();
	
	progress.setStatus("");
    }


    public boolean postArticle(ArticleRecord article) {
	boolean result = true;
	try {
	    Header header = createNNTPHeader(article);
	    String body = wrapLines(article.body, prefs.wordWrapLength);
	    NNTP.postArticle(header, body);
	} catch (IOException e) {
	    result = false;
	    Log.AddEntry("Failed to post article "+article.msgid+" \""+article.subject+"\", "+e.getMessage(), Log.TEXT, false);
	}
	return result;
    }

    String wrapLines(String lines, int max) throws IOException {
	if (max<15) max = 15;
	String result = "";
	BufferedReader reader = new BufferedReader(new StringReader(lines));
	for (String line = reader.readLine(); line != null; line = reader.readLine() ) {
	    while (line.length() > max) { 
		int pos = line.lastIndexOf(" ", max);
		if (pos == -1) pos = max;
		result = result + line.substring(0, pos) + "\n";
		line = line.substring(pos+1, line.length());
	    }
	    result = result + line + "\n";
	} 
	reader.close();
	return result;
    }

    Header createNNTPHeader(ArticleRecord article) {
	Header header = new Header();
	header.addHeaderLine("From", article.from); 
	header.addHeaderLine("Subject", article.subject);
	if (article.date!=null && !article.date.equals("")) header.addHeaderLine("Date", article.date); 
	header.addHeaderLine("Newsgroups", article.newsgroups);
	header.addHeaderLine("Message-ID", article.msgid); 
	if (article.ref!=null && !article.ref.equals("")) header.addHeaderLine("References", article.ref); 
	if (article.fup_to!=null && !article.fup_to.equals("")) header.addHeaderLine("Followup-To", article.fup_to); 
	addExtraHeaders(header, article.headers);
	header.addHeaderLine("X-Newsreader", "Yanoff Conduit v"+Version.VERSION);
    
	return header;
    }

    Header addExtraHeaders(Header header, String extraHeaderLines) {
	if (extraHeaderLines!=null && !extraHeaderLines.equals("")) {
	    String headers = new String(extraHeaderLines);
	    headers.trim(); 
	    boolean ready = false;
	    while (!ready) {
		int newline = headers.indexOf("\n");
		if (newline==-1) newline=headers.length();
		int colon = headers.indexOf(":");
		header.addHeaderLine(headers.substring(0,colon), headers.substring(colon+1,newline)); 
		if (newline < headers.length()) 
		    headers = new String(headers.substring(newline+1, headers.length()));
		else
		    ready = true;
	    }
	}
	return header;
    }

    // should be connected to mail server
    public int mailOutgoingArticles(int serverNum) throws IllegalAccessException, InstantiationException, IOException, SyncException {
	int mailedArticles = 0;

	Newsgroup outgoing = getNewsgroupByName(OUTGOING);

	for(NewsgroupRecord record=(NewsgroupRecord) outgoing.firstRecord(); record != null; record = (NewsgroupRecord) outgoing.nextRecord()) {
	    ArticleRecord article = (ArticleRecord) articles.getRecordByID(record.uid);
	    if (article.server == serverNum) {
		if (mailArticle(article)) {
		    mailedArticles++;
		    articles.deleteRecord(article);
		    outgoing.deleteRecord(record);	  
		}
	    }
	}

	return mailedArticles;
    }

    public boolean mailArticle(ArticleRecord article) throws IOException {
	boolean result = true;
	try {
	    Header header = createSMTPHeader(article);
	    String body = wrapLines(article.body, prefs.wordWrapLength);
	    SMTP.mailMessage(header, body);
	} catch (IOException e) {
	    result = false;
	    Log.AddEntry("Failed to mail article "+article.msgid+" \""+article.subject+"\", "+e.getMessage(), Log.TEXT, false);
	}
	return result;
    }

    Header createSMTPHeader(ArticleRecord article) {
	Header header = new Header();
	header.addHeaderLine("From", article.from); 
	header.addHeaderLine("Subject", article.subject);
	if (article.date!=null && !article.date.equals("")) header.addHeaderLine("Date", article.date); 
	header.addHeaderLine("To", article.newsgroups);
	header.addHeaderLine("Message-ID", article.msgid); 
	if (article.ref!=null && !article.ref.equals("")) header.addHeaderLine("References", article.ref); 
	if (article.fup_to!=null && !article.fup_to.equals("")) header.addHeaderLine("Followup-To", article.fup_to); 
	addExtraHeaders(header, article.headers);
	header.addHeaderLine("X-Mailer", "Yanoff Conduit v"+Version.VERSION);
    
	return header;
    }

    int memFree() {
	int freeMem = 0;
	try {
	    Card card = SyncManager.readSingleCardInfo(0);
	    freeMem = card.freeRam;
	} catch (SyncException e) {}
	return freeMem;
    }

    void dumpRecords(String name, PalmDatabase db){
	Log.AddEntry("Dumping all records from "+name+": ", Log.TEXT, false);
	for(PalmRecord record=db.firstRecord(); record != null; record = db.nextRecord()) {
	    Log.AddEntry(record.toFormattedString(), Log.TEXT, false);
	}
    }

    void dumpAll() throws Exception {
	Log.AddEntry(prefs.toFormattedString(), Log.TEXT, false);

	dumpRecords("Servers", servers);
	dumpRecords("Subscriptions", subscriptions);
	for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
	    // cross-links 
	    dumpRecords(NEWSGROUP_DB_PREFIX+subscription.dbid, subscription.articles);
	}
	dumpRecords("History", history);
	dumpRecords("Articles", articles);
    }

}

