/**
 * Title:        Yanoff Conduit
 * Description:  A Java conduit for the Yanoff PalmOS newsreader.
 * Copyright:    Copyright (c) Jan-Pascal van Best
 * Email:	     janpascal@vanbest.org
 * @author       Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package org.vanbest.yanoff;

import palm.conduit.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class YanoffConduit implements palm.conduit.Conduit {
  SyncProperties props = null;
  static final String NAME = "Yanoff Java Conduit";
  static final String PROPS_FILE = "yancond.props";
  static final String SUB_DIR = "yanoff";

  public YanoffConduit() {
  }

  public void open (SyncProperties props) {
    Yanoff yanoff = new Yanoff();
    this.props = props;

    Log.startSync();
    Log.AddEntry("OK Yanoff Conduit v"+Version.VERSION+" start", Log.TEXT, false);
/*    Log.out("This conduit needs Yanoff version 1.4.2b, 1.4.4 or later"); */

    ConduitConfigure config = new ConduitConfigure(NAME+" "+Version.VERSION, props.pathName+PROPS_FILE);
    config.readProps();
    Util.debugLevel = config.debugLevel & 0x00ff;

    if (config.skipSync) {
        Util.DEBUG(1,"Yanoff configured to do nothing; skipping");
    } else if (config.onceADay && !isYesterdayOrEarlier(config.lastSync)) {
        Util.DEBUG(1,"Yanoff already ran today; skipping");
    } else {
        try {
            if ((config.debugLevel & ConduitConfigure.DEBUG_SPECIAL) != 0) {
                // newsRCTest();
                // historyTest();
                // PalmDatabaseTest();
                threadTest(config);
            }
            if ((config.debugLevel & ConduitConfigure.DEBUG_SKIPNORMAL) == 0) {
                yanoff.open(config);
                try {
                  yanoff.pollAndPost();
                } finally {
                  yanoff.close();
                }
            }
            Log.AddEntry("OK Yanoff Conduit finish", Log.TEXT, false);
            Log.endSync();
        } catch(Throwable t) {
            yanoff = null;
            t.printStackTrace();
            Log.AddEntry("\nPlease do Menu/Misc/Reindex dbase in Yanoff on the Palm", Log.TEXT, false);
            Log.abortSync();
        }
    }
    config.writeProps();
    yanoff = null;
  }

  public String name() {
    return "Yanoff Conduit";
  }

  public int configure(ConfigureConduitInfo info) {
    String path = info.pathName;
    path = path + "\\" + SUB_DIR + "\\" + PROPS_FILE;

    ConduitConfigure config = new ConduitConfigure(NAME+" "+Version.VERSION, path);
    config.readProps();
    int retVal = config.createDialog();
    config.writeProps();
    info.syncNew=(config.skipSync ? SyncProperties.SYNC_DO_NOTHING : SyncProperties.SYNC_FAST);

    if(retVal == 0) return 0;
    else return 0; // or do something nasty
  }

    boolean isYesterdayOrEarlier(Date date) {
        boolean result = false;
        Calendar now = Calendar.getInstance();
        Calendar then = Calendar.getInstance(); 
        then.setTime(date);
        Log.AddEntry("then="+then.getTime().toString()+"; now="+now.getTime().toString(), Log.TEXT, false);
        if (then.get(Calendar.YEAR) < now.get(Calendar.YEAR)) {
            result = true;
        } else if (then.get(Calendar.YEAR) > now.get(Calendar.YEAR)) {
            result = false;
        } else if (then.get(Calendar.MONTH) < now.get(Calendar.MONTH)) {
            result = true;
        } else if (then.get(Calendar.MONTH) > now.get(Calendar.MONTH)) {
            result = false;
        } else if (then.get(Calendar.DAY_OF_MONTH) < now.get(Calendar.DAY_OF_MONTH)) {
            result = true;
        } else result = false;

        return result;
    }

    void newsRCTest() throws IOException {
        NewsRC newsRC = new NewsRC();
        NewsRCGroup group = newsRC.getGroup("alt.test");
        Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(1,10); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(15,20); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(12,13); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(25,30); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(21,22); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(11,11); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markRead(5,20); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markUnread(8); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markUnread(1); Log.AddEntry(group.toString(), Log.TEXT, false);
        group.markUnread(30); Log.AddEntry(group.toString(), Log.TEXT, false);
    }

    void historyTest() throws IOException {
        try {
            History history = new History(Yanoff.CREATOR_ID, Yanoff.DB_TYPE, PalmDatabase.ALLOW_CREATE);
 	    history.clearAllRecords();

//            history.addMsgid("AA"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("CC"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("EE"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("BB"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("00"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("FF"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.addMsgid("DD"); //Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);

            for (int i=0; i<10; i++) {
		long r = Math.round(Math.random()*15);
		String s = (r<10) ? "0" + r : ""+ r;
		history.addMsgid(s);
            }

            history.cleanup(true, false, 0);

//            history.removeMsgid("00"); Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.removeMsgid("CC"); Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);
//            history.removeMsgid("FF"); Log.AddEntry(history.toFormattedString()+"\n"+history.toString(), Log.TEXT, false);


            history.close();
        } catch (Exception e) {
            Log.AddEntry("An error occured: "+e.getMessage(), Log.TEXT, false);
            e.printStackTrace();
        }
    }

    void PalmDatabaseTest() {
        PalmDatabase servers = null;
        PalmDatabase articles = null;
        PalmDatabase subscriptions = null;
        History history = null;
        try {
            servers = new PalmDatabase("NewsServers", Yanoff.CREATOR_ID, Yanoff.DB_TYPE, Class.forName("org.vanbest.yanoff.ServerRecord"), PalmDatabase.READ_AHEAD);
            articles = new PalmDatabase("NewsArts", Yanoff.CREATOR_ID, Yanoff.DB_TYPE, Class.forName("org.vanbest.yanoff.ArticleRecord"), 0);
            subscriptions = new PalmDatabase("NewsSubscriptions", Yanoff.CREATOR_ID, Yanoff.DB_TYPE, Class.forName("org.vanbest.yanoff.SubscriptionRecord"), 0);

            ArticleRecord article = (ArticleRecord) articles.getRecordByIndex(1);
            ServerRecord server = (ServerRecord) servers.getRecordByIndex(1);
            SubscriptionRecord subscription = (SubscriptionRecord) subscriptions.getRecordByIndex(0);

            history = new History(Yanoff.CREATOR_ID, Yanoff.DB_TYPE, PalmDatabase.ALLOW_CREATE); 
//            history.holdPalmUpdate();

            history.addMsgid("zzz-X5");
            history.addMsgid("zzz-X6");
            history.addMsgid("zzz-X7");
            history.addMsgid("zzz-X8");
            history.addMsgid("zzz-X1");
            history.addMsgid("zzz-X0");
            history.addMsgid("zzz-X9");

            Log.out(history.toString());
//            history.close();
//
//            history = new History(Yanoff.CREATOR_ID, Yanoff.DB_TYPE, PalmDatabase.ALLOW_CREATE); 
//            Log.out("After reload: "+history.toFormattedString()+"\n"+history.toString());                        

            history.removeMsgid("zzz-X5"); Log.out("Cut out zzz-X5: \n"+history.toString());
            history.removeMsgid("zzz-X0"); Log.out("Cut out zzz-X0: \n"+history.toString());
            history.removeMsgid("zzz-X1"); Log.out("Cut out zzz-X1: \n"+history.toString());
            history.removeMsgid("zzz-X9"); Log.out("Cut out zzz-X9: \n"+history.toString());
//            Log.out("Before resume update: "+history.toFormattedString()+"\n"+history.toString());                        
//            history.resumePalmUpdate();
//            Log.out("Before reload, after resume update: "+history.toFormattedString()+"\n"+history.toString());                        

//            HistoryRecord hist = (HistoryRecord) history.getRecordByIndex(1);
//            hist.setIndex(3);
//            SyncManager.writeRec(history.DB, hist);
            history.close();

            history = new History(Yanoff.CREATOR_ID, Yanoff.DB_TYPE, PalmDatabase.ALLOW_CREATE); 
            Log.out("After reload: "+history.toFormattedString()+"\n"+history.toString());                        

            if (article != null) Log.AddEntry(article.toString(), Log.TEXT, false);
            if (server != null) Log.AddEntry(server.toString(), Log.TEXT, false);
            if (subscription != null) Log.AddEntry(subscription.toString(), Log.TEXT, false);

            Log.AddEntry(subscriptions.toString(), Log.TEXT, false);

            for(PalmRecord record=subscriptions.firstRecord(); record != null; record = subscriptions.nextRecord()) {
                Log.AddEntry(record.toString(), Log.TEXT, false);
            }

            Log.AddEntry(subscriptions.toString(), Log.TEXT, false);

            servers.close();
            articles.close();
            subscriptions.close();
            history.close();
        } catch (Throwable t) {
            Log.AddEntry("An error occured: "+t.getMessage(), Log.TEXT, false);
            t.printStackTrace();
        }
    }

    void testAddArticle(Newsgroup newsgroup, String msgid, String ref) throws IOException {
        ArticleRecord article = new ArticleRecord();
        article.from = "test@test.org";
        article.date = new Date((new Date()).getTime()- (long) Math.random()*1000*60*60*24*40).toString();
        article.subject = "Test, plz ignore";
        article.newsgroups = "alt.test.clienttest";
        article.msgid = msgid;
        article.ref = ref;
        article.body = "This is a test message.\n";
        newsgroup.articles.addRecord(article);

        NewsgroupRecord newsgroupRecord = new NewsgroupRecord(article.getId());
        newsgroup.addRecord(newsgroupRecord);
    }

    void threadTest(ConduitConfigure config) {
        PalmDatabase articles = null;
        PalmDatabase subscriptions = null;
        try {
            articles = new PalmDatabase("NewsArts", Yanoff.CREATOR_ID, Yanoff.DB_TYPE, Class.forName("org.vanbest.yanoff.ArticleRecord"), 0);
            articles.clearAllRecords();

            subscriptions = new PalmDatabase("NewsSubscriptions", Yanoff.CREATOR_ID, Yanoff.DB_TYPE, Class.forName("org.vanbest.yanoff.SubscriptionRecord"), PalmDatabase.READ_AHEAD);
            for(SubscriptionRecord subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
                Newsgroup newsgroup = new Newsgroup(Yanoff.NEWSGROUP_DB_PREFIX+subscription.dbid, Yanoff.CREATOR_ID, Yanoff.DB_TYPE, 
		        Class.forName(Yanoff.NEWSGROUP_CLASS), 0, articles);
                subscription.articles = newsgroup;
                subscription.articles.clearAllRecords();
            }

            SubscriptionRecord subscription = (SubscriptionRecord) subscriptions.getRecordByIndex(1);
            Newsgroup newsgroup = subscription.articles;

            testAddArticle(newsgroup, "<A12>", "<A> <A1>");
            testAddArticle(newsgroup, "<Z>", "");
            testAddArticle(newsgroup, "<A13>", "<A><A1>");
            testAddArticle(newsgroup, "<A>", "");
            testAddArticle(newsgroup, "<A1>", "<A>");
            testAddArticle(newsgroup, "<A11>", "<A><A1>");
            testAddArticle(newsgroup, "<X>", "");
            testAddArticle(newsgroup, "<A131>", "<A> <A1> <A13>");
            testAddArticle(newsgroup, "<A2>", "<A>");
            testAddArticle(newsgroup, "<A14>", "<A> <A1>");
            testAddArticle(newsgroup, "<Y>", "");

            ConduitProgress progress = new ConduitProgress(config);
            progress.createDialog();
            progress.setStatus("");
            Threader threader = new Threader();
            threader.thread(newsgroup, articles, progress);
            progress.close();
            Log.out(threader.toString());

            newsgroup.clearAllRecords();
            for (Enumeration e=threader.order(); e.hasMoreElements(); ) {
                ArticleRecord article = (ArticleRecord) e.nextElement();
                Log.out(article.msgid + article.subject);
                NewsgroupRecord record = new NewsgroupRecord(article.getId());
                newsgroup.addRecord(record);
            }

            for(subscription=(SubscriptionRecord) subscriptions.firstRecord(); subscription != null; subscription = (SubscriptionRecord) subscriptions.nextRecord()) {
                subscription.articles.close();
            }
            articles.close();
            subscriptions.close();
        } catch (Throwable t) {
            Log.AddEntry("An error occured: "+t.getMessage(), Log.TEXT, false);
            t.printStackTrace();
        }
    }

}

